@@ -365,6 +365,35 @@ qcdm_cmd_esn_result (const char *buf, size_t len, int *out_error)
 
 /**********************************************************************/
 
+size_t
+qcdm_cmd_control_new (char *buf, size_t len, u_int8_t mode)
+{
+    char cmdbuf[5];
+    DMCmdControl *cmd = (DMCmdControl *) &cmdbuf[0];
+
+    qcdm_return_val_if_fail (buf != NULL, 0);
+    qcdm_return_val_if_fail (len >= sizeof (*cmd) + DIAG_TRAILER_LEN, 0);
+
+    memset (cmd, 0, sizeof (*cmd));
+    cmd->code = DIAG_CMD_CONTROL;
+    cmd->mode = htole16 ((u_int16_t) mode);
+
+    return dm_encapsulate_buffer (cmdbuf, sizeof (*cmd), sizeof (cmdbuf), buf, len);
+}
+
+QcdmResult *
+qcdm_cmd_control_result (const char *buf, size_t len, int *out_error)
+{
+    qcdm_return_val_if_fail (buf != NULL, NULL);
+
+    if (!check_command (buf, len, DIAG_CMD_CONTROL, sizeof (DMCmdControl), out_error))
+        return NULL;
+
+    return qcdm_result_new ();
+}
+
+/**********************************************************************/
+
 size_t
 qcdm_cmd_cdma_status_new (char *buf, size_t len)
 {
@@ -623,7 +652,7 @@ qcdm_cmd_pilot_sets_result (const char *buf, size_t len, int *out_error)
     sets_len = rsp->candidate_count * sizeof (DMCmdPilotSetsSet);
     if (sets_len > 0) {
         qcdm_result_add_u8_array (result,
-                                  PILOT_SETS_CMD_ACTIVE_SET,
+                                  PILOT_SETS_CMD_CANDIDATE_SET,
                                   (const u_int8_t *) &rsp->sets[rsp->active_count],
                                   sets_len);
     }
@@ -631,7 +660,7 @@ qcdm_cmd_pilot_sets_result (const char *buf, size_t len, int *out_error)
     sets_len = rsp->neighbor_count * sizeof (DMCmdPilotSetsSet);
     if (sets_len > 0) {
         qcdm_result_add_u8_array (result,
-                                  PILOT_SETS_CMD_ACTIVE_SET,
+                                  PILOT_SETS_CMD_NEIGHBOR_SET,
                                   (const u_int8_t *) &rsp->sets[rsp->active_count + rsp->candidate_count],
                                   sets_len);
     }
@@ -653,7 +682,7 @@ qcdm_cmd_pilot_sets_result_get_num (QcdmResult *result,
     set_name = set_num_to_str (set_type);
     qcdm_return_val_if_fail (set_name != NULL, FALSE);
 
-    if (!qcdm_result_get_u8_array (result, set_name, &array, &array_len))
+    if (qcdm_result_get_u8_array (result, set_name, &array, &array_len))
         return FALSE;
 
     *out_num = array_len / sizeof (DMCmdPilotSetsSet);
@@ -678,7 +707,7 @@ qcdm_cmd_pilot_sets_result_get_pilot (QcdmResult *result,
     set_name = set_num_to_str (set_type);
     qcdm_return_val_if_fail (set_name != NULL, FALSE);
 
-    if (!qcdm_result_get_u8_array (result, set_name, &array, &array_len))
+    if (qcdm_result_get_u8_array (result, set_name, &array, &array_len))
         return FALSE;
 
     qcdm_return_val_if_fail (num < array_len / sizeof (DMCmdPilotSetsSet), FALSE);
@@ -1445,16 +1474,16 @@ qcdm_cmd_nw_subsys_modem_snapshot_cdma_new (char *buf,
     cmd->hdr.code = DIAG_CMD_SUBSYS;
     switch (chipset) {
     case QCDM_NW_CHIPSET_6500:
-        cmd->hdr.subsys_id = DIAG_SUBSYS_NW_CONTROL_6500;
+        cmd->hdr.subsys_id = DIAG_SUBSYS_NOVATEL_6500;
         break;
     case QCDM_NW_CHIPSET_6800:
-        cmd->hdr.subsys_id = DIAG_SUBSYS_NW_CONTROL_6800;
+        cmd->hdr.subsys_id = DIAG_SUBSYS_NOVATEL_6800;
         break;
     default:
         qcdm_assert_not_reached ();
     }
-    cmd->hdr.subsys_cmd = htole16 (DIAG_SUBSYS_NW_CONTROL_MODEM_SNAPSHOT);
-    cmd->technology = DIAG_SUBSYS_NW_CONTROL_MODEM_SNAPSHOT_TECH_CDMA_EVDO;
+    cmd->hdr.subsys_cmd = htole16 (DIAG_SUBSYS_NOVATEL_MODEM_SNAPSHOT);
+    cmd->technology = DIAG_SUBSYS_NOVATEL_MODEM_SNAPSHOT_TECH_CDMA_EVDO;
     cmd->snapshot_mask = htole32 (0xFFFF);
 
     return dm_encapsulate_buffer (cmdbuf, sizeof (*cmd), sizeof (cmdbuf), buf, len);
@@ -1507,6 +1536,76 @@ qcdm_cmd_nw_subsys_modem_snapshot_cdma_result (const char *buf, size_t len, int
 
 /**********************************************************************/
 
+size_t
+qcdm_cmd_nw_subsys_eri_new (char *buf,
+                            size_t len,
+                            u_int8_t chipset)
+{
+    char cmdbuf[sizeof (DMCmdSubsysHeader) + 2];
+    DMCmdSubsysHeader *cmd = (DMCmdSubsysHeader *) &cmdbuf[0];
+
+    qcdm_return_val_if_fail (buf != NULL, 0);
+    qcdm_return_val_if_fail (len >= sizeof (*cmd) + DIAG_TRAILER_LEN, 0);
+
+    /* Validate chipset */
+    if (chipset != QCDM_NW_CHIPSET_6500 && chipset != QCDM_NW_CHIPSET_6800) {
+        qcdm_err (0, "Unknown Novatel chipset 0x%X", chipset);
+        return 0;
+    }
+
+    memset (cmd, 0, sizeof (*cmd));
+    cmd->code = DIAG_CMD_SUBSYS;
+    switch (chipset) {
+    case QCDM_NW_CHIPSET_6500:
+        cmd->subsys_id = DIAG_SUBSYS_NOVATEL_6500;
+        break;
+    case QCDM_NW_CHIPSET_6800:
+        cmd->subsys_id = DIAG_SUBSYS_NOVATEL_6800;
+        break;
+    default:
+        qcdm_assert_not_reached ();
+    }
+    cmd->subsys_cmd = htole16 (DIAG_SUBSYS_NOVATEL_ERI);
+
+    return dm_encapsulate_buffer (cmdbuf, sizeof (*cmd), sizeof (cmdbuf), buf, len);
+}
+
+QcdmResult *
+qcdm_cmd_nw_subsys_eri_result (const char *buf, size_t len, int *out_error)
+{
+    QcdmResult *result = NULL;
+    DMCmdSubsysNwEriRsp *rsp = (DMCmdSubsysNwEriRsp *) buf;
+    char str[50];
+
+    qcdm_return_val_if_fail (buf != NULL, NULL);
+
+    if (!check_command (buf, len, DIAG_CMD_SUBSYS, sizeof (DMCmdSubsysNwEriRsp), out_error))
+        return NULL;
+
+    /* FIXME: check 'status' when we know what it means */
+
+    result = qcdm_result_new ();
+
+    qcdm_result_add_u8 (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_ROAM, rsp->roam);
+    qcdm_result_add_u8 (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_INDICATOR_ID, rsp->indicator_id);
+    qcdm_result_add_u8 (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_ICON_ID, rsp->icon_id);
+    qcdm_result_add_u8 (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_ICON_MODE, rsp->icon_mode);
+    qcdm_result_add_u8 (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_CALL_PROMPT_ID, rsp->call_prompt_id);
+    qcdm_result_add_u8 (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_ALERT_ID, rsp->alert_id);
+
+    qcdm_warn_if_fail (rsp->text_len < sizeof (str));
+    if (rsp->text_len < sizeof (str)) {
+        qcdm_assert (sizeof (str) > sizeof (rsp->text));
+        memcpy (str, rsp->text, sizeof (rsp->text));
+        str[rsp->text_len] = '\0';
+        qcdm_result_add_string (result, QCDM_CMD_NW_SUBSYS_ERI_ITEM_TEXT, str);
+    }
+
+    return result;
+}
+
+/**********************************************************************/
+
 static size_t
 qcdm_cmd_log_config_new (char *buf,
                          size_t len,
