@@ -66,6 +66,19 @@ enum {
     QCDM_HDR_REV_A = 0x02
 };
 
+enum {
+    QCDM_ERI_ROAMING_ICON_ON = 0,
+    QCDM_ERI_ROAMING_ICON_OFF = 1,
+    QCDM_ERI_ROAMING_ICON_FLASH = 2,
+    /* Values greater than 2 are OEM defined */
+};
+
+enum {
+    /* Valid with QCDM_ERI_ROAMING_ICON_FLASH and greater */
+    QCDM_ERI_ROAMING_ICON_MODE_NORMAL = 0,
+    QCDM_ERI_ROAMING_ICON_MODE_FLASH = 1,
+};
+
 /**********************************************************************/
 
 #define QCDM_CMD_VERSION_INFO_ITEM_COMP_DATE "comp-date"
@@ -92,6 +105,19 @@ QcdmResult *qcdm_cmd_esn_result (const char *buf,
 
 /**********************************************************************/
 
+enum {
+    QCDM_CMD_CONTROL_MODE_OFFLINE = 1,
+    QCDM_CMD_CONTROL_MODE_RESET = 2,
+};
+
+size_t      qcdm_cmd_control_new    (char *buf, size_t len, u_int8_t mode);
+
+QcdmResult *qcdm_cmd_control_result (const char *buf,
+                                     size_t len,
+                                     int *out_error);
+
+/**********************************************************************/
+
 /* Values for QCDM_CMD_CDMA_STATUS_ITEM_RF_MODE */
 enum {
     QCDM_CMD_CDMA_STATUS_RF_MODE_ANALOG = 0,
@@ -146,6 +172,10 @@ QcdmResult *qcdm_cmd_sw_version_result (const char *buf,
 
 /**********************************************************************/
 
+#define QCDM_CMD_STATUS_SNAPSHOT_ITEM_ESN                "esn"
+
+#define QCDM_CMD_STATUS_SNAPSHOT_ITEM_HOME_MCC           "mcc"
+
 /* One of QCDM_CDMA_BAND_CLASS_* */
 #define QCDM_CMD_STATUS_SNAPSHOT_ITEM_BAND_CLASS         "band-class"
 
@@ -270,6 +300,7 @@ enum {
     QCDM_CMD_NV_MODE_PREF_ITEM_MODE_PREF_GSM_UMTS_ONLY   = 0x11,
     QCDM_CMD_NV_MODE_PREF_ITEM_MODE_PREF_1X_HDR_ONLY     = 0x13,
     QCDM_CMD_NV_MODE_PREF_ITEM_MODE_PREF_LTE_ONLY        = 0x1E,
+    QCDM_CMD_NV_MODE_PREF_ITEM_MODE_PREF_GSM_UMTS_LTE_ONLY = 0x1F,
     QCDM_CMD_NV_MODE_PREF_ITEM_MODE_PREF_1X_HDR_LTE_ONLY = 0x24,
 };
 
@@ -295,6 +326,29 @@ QcdmResult *qcdm_cmd_nv_set_mode_pref_result (const char *buf,
 
 /**********************************************************************/
 
+enum {
+    QCDM_CMD_NV_HYBRID_PREF_ITEM_REV_HYBRID_OFF = 0x00,
+    QCDM_CMD_NV_HYBRID_PREF_ITEM_REV_HYBRID_ON = 0x01,
+};
+
+#define QCDM_CMD_NV_GET_HYBRID_PREF_ITEM_HYBRID_PREF "hybrid-pref"
+
+size_t      qcdm_cmd_nv_get_hybrid_pref_new    (char *buf, size_t len);
+
+QcdmResult *qcdm_cmd_nv_get_hybrid_pref_result (const char *buf,
+                                                size_t len,
+                                                int *out_error);
+
+size_t      qcdm_cmd_nv_set_hybrid_pref_new    (char *buf,
+                                                size_t len,
+                                                u_int8_t hybrid_pref);
+
+QcdmResult *qcdm_cmd_nv_set_hybrid_pref_result (const char *buf,
+                                                size_t len,
+                                                int *out_error);
+
+/**********************************************************************/
+
 /* Values for QCDM_CMD_NV_GET_HDR_REV_PREF_ITEM_REV_PREF */
 enum {
     QCDM_CMD_NV_HDR_REV_PREF_ITEM_REV_PREF_0 = 0x00,
@@ -378,6 +432,8 @@ enum {
     QCDM_CMD_CM_SUBSYS_STATE_INFO_MODE_PREF_WCDMA_ONLY = 0x0E,
     QCDM_CMD_CM_SUBSYS_STATE_INFO_MODE_PREF_PERSISTENT_MODE = 0x0F,
     QCDM_CMD_CM_SUBSYS_STATE_INFO_MODE_PREF_NO_CHANGE = 0x10,
+    QCDM_CMD_CM_SUBSYS_STATE_INFO_MODE_PREF_LTE_ONLY = 0x26,
+    QCDM_CMD_CM_SUBSYS_STATE_INFO_MODE_PREF_GSM_WCDMA_LTE_ONLY = 0x27,
 };
 
 #define QCDM_CMD_CM_SUBSYS_STATE_INFO_ITEM_CALL_STATE             "call-state"
@@ -578,6 +634,32 @@ QcdmResult *qcdm_cmd_nw_subsys_modem_snapshot_cdma_result (const char *buf,
 
 /**********************************************************************/
 
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_ROAM           "roam"
+
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_INDICATOR_ID   "indicator-id"
+
+/* One of QCDM_ERI_ROAMING_ICON_* */
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_ICON_ID        "icon-id"
+
+/* One of QCDM_ERI_ROAMING_ICON_MODE_* */
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_ICON_MODE      "icon-mode"
+
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_CALL_PROMPT_ID "call-prompt-id"
+
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_ALERT_ID       "alert-id"
+
+#define QCDM_CMD_NW_SUBSYS_ERI_ITEM_TEXT           "text"
+
+size_t      qcdm_cmd_nw_subsys_eri_new    (char *buf,
+                                           size_t len,
+                                           u_int8_t chipset);
+
+QcdmResult *qcdm_cmd_nw_subsys_eri_result (const char *buf,
+                                           size_t len,
+                                           int *out_error);
+
+/**********************************************************************/
+
 #define QCDM_CMD_WCDMA_SUBSYS_STATE_INFO_ITEM_IMEI     "imei"
 
 #define QCDM_CMD_WCDMA_SUBSYS_STATE_INFO_ITEM_IMSI     "imsi"
