@@ -29,6 +29,7 @@
 #include "mm-serial-port.h"
 #include "mm-serial-parsers.h"
 #include "mm-log.h"
+#include "mm-modem-helpers.h"
 
 static void modem_init (MMModem *modem_class);
 
@@ -62,60 +63,6 @@ mm_modem_anydata_cdma_new (const char *device,
 
 /*****************************************************************************/
 
-static const char *
-strip_response (const char *resp, const char *cmd)
-{
-    const char *p = resp;
-
-    if (p) {
-        if (!strncmp (p, cmd, strlen (cmd)))
-            p += strlen (cmd);
-        while (*p == ' ')
-            p++;
-    }
-    return p;
-}
-
-static gboolean
-uint_from_match_item (GMatchInfo *match_info, guint32 num, guint32 *val)
-{
-    long int tmp;
-    char *str;
-    gboolean success = FALSE;
-
-    str = g_match_info_fetch (match_info, num);
-    g_return_val_if_fail (str != NULL, FALSE);
-
-    errno = 0;
-    tmp = strtol (str, NULL, 10);
-    if (errno == 0 && tmp >= 0 && tmp <= G_MAXUINT) {
-        *val = (guint32) tmp;
-        success = TRUE;
-    }
-    g_free (str);
-    return success;
-}
-
-static gboolean
-int_from_match_item (GMatchInfo *match_info, guint32 num, gint *val)
-{
-    long int tmp;
-    char *str;
-    gboolean success = FALSE;
-
-    str = g_match_info_fetch (match_info, num);
-    g_return_val_if_fail (str != NULL, FALSE);
-
-    errno = 0;
-    tmp = strtol (str, NULL, 10);
-    if (errno == 0 && tmp >= G_MININT && tmp <= G_MAXINT) {
-        *val = (gint) tmp;
-        success = TRUE;
-    }
-    g_free (str);
-    return success;
-}
-
 static void
 evdo_state_done (MMAtSerialPort *port,
                  GString *response,
@@ -139,7 +86,7 @@ evdo_state_done (MMAtSerialPort *port,
         return;
     }
 
-    reply = strip_response (response->str, "*HSTATE:");
+    reply = mm_strip_tag (response->str, "*HSTATE:");
 
     /* Format is "<at state>,<session state>,<channel>,<pn>,<EcIo>,<rssi>,..." */
     r = g_regex_new ("\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*([^,\\)]*)\\s*,\\s*([^,\\)]*)\\s*,.*",
@@ -156,10 +103,10 @@ evdo_state_done (MMAtSerialPort *port,
         gint dbm = 0;
 
         /* dBm is between -106 (worst) and -20.7 (best) */
-        int_from_match_item (match_info, 6, &dbm);
+        mm_int_from_match_item (match_info, 6, &dbm);
 
         /* Parse the EVDO radio state */
-        if (uint_from_match_item (match_info, 1, &val)) {
+        if (mm_uint_from_match_item (match_info, 1, &val)) {
             switch (val) {
             case 3:  /* IDLE */
                 /* If IDLE and the EVDO dBm is -105 or lower, assume no service.
@@ -214,7 +161,7 @@ state_done (MMAtSerialPort *port,
         return;
     }
 
-    reply = strip_response (response->str, "*STATE:");
+    reply = mm_strip_tag (response->str, "*STATE:");
 
     /* Format is "<channel>,<pn>,<sid>,<nid>,<state>,<rssi>,..." */
     r = g_regex_new ("\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*(\\d+)\\s*,\\s*([^,\\)]*)\\s*,.*",
@@ -231,10 +178,10 @@ state_done (MMAtSerialPort *port,
         gint dbm = 0;
 
         /* dBm is between -106 (worst) and -20.7 (best) */
-        int_from_match_item (match_info, 6, &dbm);
+        mm_int_from_match_item (match_info, 6, &dbm);
 
         /* Parse the 1x radio state */
-        if (uint_from_match_item (match_info, 5, &val)) {
+        if (mm_uint_from_match_item (match_info, 5, &val)) {
             switch (val) {
             case 1:  /* IDLE */
                 /* If IDLE and the 1X dBm is -105 or lower, assume no service.
