@@ -32,12 +32,15 @@
 #include "mm-at-serial-port.h"
 #include "mm-generic-gsm.h"
 #include "mm-modem-helpers.h"
+#include "mm-utils.h"
 #include "mm-log.h"
 
 struct _MMModemIceraPrivate {
     /* Pending connection attempt */
     MMCallbackInfo *connect_pending_data;
     guint connect_pending_id;
+    guint configure_context_id;
+    guint configure_context_tries;
 
     char *username;
     char *password;
@@ -47,6 +50,24 @@ struct _MMModemIceraPrivate {
 
 #define MM_MODEM_ICERA_GET_PRIVATE(m) (MM_MODEM_ICERA_GET_INTERFACE (m)->get_private(m))
 
+static void connect_pending_done (MMModemIcera *self);
+static void icera_call_control (MMModemIcera *self,
+                                guint32 cid,
+                                gboolean activate,
+                                MMAtSerialResponseFn callback,
+                                gpointer user_data);
+
+static void
+cleanup_configure_context (MMModemIcera *self)
+{
+    MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
+
+    if (priv->configure_context_id != 0) {
+        g_source_remove (priv->configure_context_id);
+        priv->configure_context_id = 0;
+    }
+}
+
 static void
 get_allowed_mode_done (MMAtSerialPort *port,
                        GString *response,
@@ -200,6 +221,8 @@ nwstate_to_act (const char *str)
         return MM_MODEM_GSM_ACCESS_TECH_HSUPA;
     else if (!strcmp (str, "3G-HSDPA-HSUPA") || !strcmp (str, "HSDPA-HSUPA"))
         return MM_MODEM_GSM_ACCESS_TECH_HSPA;
+    else if (!strcmp (str, "3G-HSDPA-HSUPA-HSPA+") || !strcmp (str, "HSDPA-HSUPA-HSPA+"))
+        return MM_MODEM_GSM_ACCESS_TECH_HSPA_PLUS;
 
     return MM_MODEM_GSM_ACCESS_TECH_UNKNOWN;
 }
@@ -298,10 +321,10 @@ mm_modem_icera_get_access_technology (MMModemIcera *self,
 /****************************************************************/
 
 static void
-disconnect_ipdpact_done (MMAtSerialPort *port,
-                         GString *response,
-                         GError *error,
-                         gpointer user_data)
+disconnect_done (MMAtSerialPort *port,
+                 GString *response,
+                 GError *error,
+                 gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
 
@@ -320,17 +343,14 @@ mm_modem_icera_do_disconnect (MMGenericGsm *gsm,
                               gpointer user_data)
 {
     MMCallbackInfo *info;
-    MMAtSerialPort *primary;
-    char *command;
+
+    /* Cleanup any running connect stuff */
+    cleanup_configure_context (MM_MODEM_ICERA (gsm));
+    connect_pending_done (MM_MODEM_ICERA (gsm));
 
     info = mm_callback_info_new (MM_MODEM (gsm), callback, user_data);
 
-    primary = mm_generic_gsm_get_at_port (gsm, MM_AT_PORT_FLAG_PRIMARY);
-    g_assert (primary);
-
-    command = g_strdup_printf ("%%IPDPACT=%d,0", cid);
-    mm_at_serial_port_queue_command (primary, command, 3, disconnect_ipdpact_done, info);
-    g_free (command);
+    icera_call_control (MM_MODEM_ICERA (gsm), cid, FALSE, disconnect_done, info);
 }
 
 /*****************************************************************************/
@@ -474,6 +494,7 @@ _get_cid (MMModemIcera *self)
 
 static void
 icera_call_control (MMModemIcera *self,
+                    guint32 cid,
                     gboolean activate,
                     MMAtSerialResponseFn callback,
                     gpointer user_data)
@@ -484,8 +505,13 @@ icera_call_control (MMModemIcera *self,
     primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
-    command = g_strdup_printf ("%%IPDPACT=%d,%d", _get_cid (self), activate ? 1 : 0);
-    mm_at_serial_port_queue_command (primary, command, 3, callback, user_data);
+    /* Firmware might have a custom call control command (ie, Sierra) */
+    if (MM_MODEM_ICERA_GET_INTERFACE (self)->get_call_control_cmd)
+        command = MM_MODEM_ICERA_GET_INTERFACE (self)->get_call_control_cmd (self, cid, activate);
+    else
+        command = g_strdup_printf ("%%IPDPACT=%d,%d", cid, activate ? 1 : 0);
+
+    mm_at_serial_port_queue_command (primary, command, 15, callback, user_data);
     g_free (command);
 }
 
@@ -513,7 +539,7 @@ icera_connect_timed_out (gpointer data)
                                            "Connection timed out");
     }
 
-    icera_call_control (self, FALSE, timeout_done, self);
+    icera_call_control (self, _get_cid (self), FALSE, timeout_done, self);
     return FALSE;
 }
 
@@ -542,6 +568,15 @@ icera_connected (MMAtSerialPort *port,
 
         priv->connect_pending_data = info;
         priv->connect_pending_id = g_timeout_add_seconds (30, icera_connect_timed_out, self);
+
+        /* If the implementor has a custom call control command, then assume
+         * that it returns OK only when the connection is ready, and thus all
+         * we have to do is complete the connection request. Otherwise, if
+         * using standard Icera commands, we wait for the connection indication
+         * via %IPDPACT unsolicited messages.
+         */
+        if (MM_MODEM_ICERA_GET_INTERFACE (self)->get_call_control_cmd)
+            connect_pending_done (self);
     }
 }
 
@@ -559,7 +594,29 @@ old_context_clear_done (MMAtSerialPort *port,
         return;
 
     /* Activate the PDP context and start the data session */
-    icera_call_control (MM_MODEM_ICERA (info->modem), TRUE, icera_connected, info);
+    icera_call_control (MM_MODEM_ICERA (info->modem),
+                        _get_cid (MM_MODEM_ICERA (info->modem)),
+                        TRUE,
+                        icera_connected,
+                        info);
+}
+
+static void configure_context (MMAtSerialPort *port, MMCallbackInfo *info,
+                               char *username, char *password, gint cid);
+
+static gboolean
+retry_config_context (gpointer data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) data;
+    MMModemIcera *self = MM_MODEM_ICERA (info->modem);
+    MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
+    MMAtSerialPort *primary;
+
+    priv->configure_context_id = 0;
+    primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
+    g_assert (primary);
+    configure_context (primary, info, priv->username, priv->password, _get_cid (self));
+    return FALSE;
 }
 
 static void
@@ -569,20 +626,51 @@ auth_done (MMAtSerialPort *port,
            gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    MMModemIcera *self = MM_MODEM_ICERA (info->modem);
+    MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
 
     /* If the modem has already been removed, return without
      * scheduling callback */
     if (mm_callback_info_check_modem_removed (info))
         return;
 
-    if (error)
+    if (error) {
+        /* Retry configuring the context. It sometimes fails with a 583
+         * error ["a profile (CID) is currently active"] if a connect
+         * is attempted too soon after a disconnect. */
+        if (++priv->configure_context_tries < 3) {
+            priv->configure_context_id =
+                    g_timeout_add_seconds (1, retry_config_context, info);
+            return;
+        }
         mm_generic_gsm_connect_complete (MM_GENERIC_GSM (info->modem), error, info);
-    else {
+    } else {
         /* Ensure the PDP context is deactivated */
-        icera_call_control (MM_MODEM_ICERA (info->modem), FALSE, old_context_clear_done, info);
+        icera_call_control (self, _get_cid (self), FALSE, old_context_clear_done, info);
     }
 }
 
+static void
+configure_context(MMAtSerialPort *port, MMCallbackInfo *info,
+                  char *username, char *password, gint cid)
+{
+    char *command;
+
+    /* Both user and password are required; otherwise firmware returns an error */
+    if (!username || !password)
+		command = g_strdup_printf ("%%IPDPCFG=%d,0,0,\"\",\"\"", cid);
+    else {
+        command = g_strdup_printf ("%%IPDPCFG=%d,0,1,\"%s\",\"%s\"",
+                                   cid,
+                                   username ? username : "",
+                                   password ? password : "");
+
+    }
+
+    mm_at_serial_port_queue_command (port, command, 3, auth_done, info);
+    g_free (command);
+}
+
 void
 mm_modem_icera_do_connect (MMModemIcera *self,
                            const char *number,
@@ -593,8 +681,6 @@ mm_modem_icera_do_connect (MMModemIcera *self,
     MMModemIceraPrivate *priv = MM_MODEM_ICERA_GET_PRIVATE (self);
     MMCallbackInfo *info;
     MMAtSerialPort *primary;
-    gint cid;
-    char *command;
 
     mm_modem_set_state (modem, MM_MODEM_STATE_CONNECTING, MM_MODEM_STATE_REASON_NONE);
 
@@ -603,22 +689,8 @@ mm_modem_icera_do_connect (MMModemIcera *self,
     primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (self), MM_AT_PORT_FLAG_PRIMARY);
     g_assert (primary);
 
-    cid = _get_cid (self);
-
-
-    /* Both user and password are required; otherwise firmware returns an error */
-    if (!priv->username || !priv->password)
-		command = g_strdup_printf ("%%IPDPCFG=%d,0,0,\"\",\"\"", cid);
-    else {
-        command = g_strdup_printf ("%%IPDPCFG=%d,0,1,\"%s\",\"%s\"",
-                                   cid,
-                                   priv->username ? priv->username : "",
-                                   priv->password ? priv->password : "");
-
-    }
-
-    mm_at_serial_port_queue_command (primary, command, 3, auth_done, info);
-    g_free (command);
+    priv->configure_context_tries = 0;
+    configure_context (primary, info, priv->username, priv->password, _get_cid (self));
 }
 
 /****************************************************************/
@@ -637,6 +709,8 @@ ip4_config_invoke (MMCallbackInfo *info)
 
     callback (info->modem,
               GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-address")),
+              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-netmask")),
+              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-gateway")),
               (GArray *) mm_callback_info_get_data (info, "ip4-dns"),
               info->error, info->user_data);
 }
@@ -673,7 +747,11 @@ get_ip4_config_done (MMAtSerialPort *port,
     cid = _get_cid (MM_MODEM_ICERA (info->modem));
     dns_array = g_array_sized_new (FALSE, TRUE, sizeof (guint32), 2);
 
-    /* %IPDPADDR: <cid>,<ip>,<gw>,<dns1>,<dns2>[,<nbns1>,<nbns2>] */
+    /* %IPDPADDR: <cid>,<ip>,<gw>,<dns1>,<dns2>[,<nbns1>,<nbns2>[,<??>,<netmask>,<gw>]]
+     *
+     * Sierra USB305: %IPDPADDR: 2, 21.93.217.11, 21.93.217.10, 10.177.0.34, 10.161.171.220, 0.0.0.0, 0.0.0.0
+     * K3805-Z: %IPDPADDR: 2, 21.93.217.11, 21.93.217.10, 10.177.0.34, 10.161.171.220, 0.0.0.0, 0.0.0.0, 255.0.0.0, 255.255.255.0, 21.93.217.10,
+     */
     items = g_strsplit (response->str + strlen (IPDPADDR_TAG), ", ", 0);
 
     for (iter = items, i = 0; *iter; iter++, i++) {
@@ -691,12 +769,23 @@ get_ip4_config_done (MMAtSerialPort *port,
         } else if (i == 1) { /* IP address */
             if (inet_pton (AF_INET, *iter, &tmp) > 0)
                 mm_callback_info_set_data (info, "ip4-address", GUINT_TO_POINTER (tmp), NULL);
+        } else if (i == 2) { /* Gateway */
+            if ((inet_pton (AF_INET, *iter, &tmp) > 0) && (tmp > 0))
+                mm_callback_info_set_data (info, "ip4-gateway", GUINT_TO_POINTER (tmp), NULL);
         } else if (i == 3) { /* DNS 1 */
             if (inet_pton (AF_INET, *iter, &tmp) > 0)
                 g_array_append_val (dns_array, tmp);
         } else if (i == 4) { /* DNS 2 */
             if (inet_pton (AF_INET, *iter, &tmp) > 0)
                 g_array_append_val (dns_array, tmp);
+        } else if (i == 8) { /* Netmask */
+            if (inet_pton (AF_INET, *iter, &tmp) > 0)
+                mm_callback_info_set_data (info, "ip4-netmask", GUINT_TO_POINTER (tmp), NULL);
+        } else if (i == 9) { /* Duplicate gateway */
+            if (mm_callback_info_get_data (info, "ip4-gateway") == NULL) {
+                if (inet_pton (AF_INET, *iter, &tmp) > 0)
+                    mm_callback_info_set_data (info, "ip4-gateway", GUINT_TO_POINTER (tmp), NULL);
+            }
         }
     }
 
@@ -822,6 +911,438 @@ mm_modem_icera_get_local_timestamp (MMModemIcera *self,
 
 /****************************************************************/
 
+typedef struct {
+    MMModemGsmBand band;
+    char *name;
+    gboolean enabled;
+    gpointer data;
+} Band;
+
+static void
+band_free (Band *b)
+{
+    g_free (b->name);
+    g_free (b);
+}
+
+static const Band modem_bands[] = {
+    /* Sort 3G first since it's preferred */
+    { MM_MODEM_GSM_BAND_U2100, "FDD_BAND_I",    FALSE, NULL },
+    { MM_MODEM_GSM_BAND_U1900, "FDD_BAND_II",   FALSE, NULL },
+    { MM_MODEM_GSM_BAND_U1800, "FDD_BAND_III",  FALSE, NULL },
+    { MM_MODEM_GSM_BAND_U17IV, "FDD_BAND_IV",   FALSE, NULL },
+    { MM_MODEM_GSM_BAND_U800,  "FDD_BAND_VI",   FALSE, NULL },
+    { MM_MODEM_GSM_BAND_U850,  "FDD_BAND_V",    FALSE, NULL },
+    { MM_MODEM_GSM_BAND_U900,  "FDD_BAND_VIII", FALSE, NULL },
+    /* 2G second */
+    { MM_MODEM_GSM_BAND_G850,  "G850",          FALSE, NULL },
+    { MM_MODEM_GSM_BAND_DCS,   "DCS",           FALSE, NULL },
+    { MM_MODEM_GSM_BAND_EGSM,  "EGSM",          FALSE, NULL },
+    { MM_MODEM_GSM_BAND_PCS,   "PCS",           FALSE, NULL },
+    /* And ANY last since it's most inclusive */
+    { MM_MODEM_GSM_BAND_ANY,   "ANY",           FALSE, NULL },
+};
+
+static MMModemGsmBand
+icera_band_to_mm (const char *icera)
+{
+    int i;
+
+    for (i = 0 ; i < G_N_ELEMENTS (modem_bands); i++) {
+        if (g_strcmp0 (icera, modem_bands[i].name) == 0)
+            return modem_bands[i].band;
+    }
+    return MM_MODEM_GSM_BAND_UNKNOWN;
+}
+
+static const char *
+mm_band_to_icera (MMModemGsmBand band)
+{
+    int i;
+
+    for (i = 0; i < G_N_ELEMENTS (modem_bands); i++) {
+        if (modem_bands[i].band == band)
+            return modem_bands[i].name;
+    }
+    return NULL;
+}
+
+/* returns a list of 'Band' structs */
+static GSList *
+build_bands_info (const gchar *response, gboolean build_command)
+{
+    GSList *bands = NULL;
+    GRegex *r;
+    GMatchInfo *match;
+
+    /*
+     * Response is a number of lines of the form:
+     *   "EGSM": 0
+     *   "FDD_BAND_I": 1
+     *   ...
+     * with 1 and 0 indicating whether the particular band is enabled or not.
+     */
+    r = g_regex_new ("^\"(\\w+)\": (\\d)",
+                     G_REGEX_MULTILINE, G_REGEX_MATCH_NEWLINE_ANY,
+                     NULL);
+    g_assert (r != NULL);
+
+    g_regex_match (r, response, 0, &match);
+    while (g_match_info_matches (match)) {
+        gchar *name, *enabled;
+        Band *b;
+        MMModemGsmBand band;
+
+        name = g_match_info_fetch (match, 1);
+        enabled = g_match_info_fetch (match, 2);
+        band = icera_band_to_mm (name);
+        if (band != MM_MODEM_GSM_BAND_UNKNOWN && band != MM_MODEM_GSM_BAND_ANY) {
+            b = g_malloc0 (sizeof (Band));
+            b->band = band;
+            b->enabled = (enabled[0] == '1' ? TRUE : FALSE);
+            if (build_command) {
+                /* abuse 'name' for the AT command to check band support */
+                b->name = g_strdup_printf ("%%IPBM=\"%s\",%c",
+                                           name,
+                                           b->enabled ? '1' : '0');
+            }
+            bands = g_slist_append (bands, b);
+        }
+        g_free (name);
+        g_free (enabled);
+        g_match_info_next (match, NULL);
+    }
+    g_match_info_free (match);
+    g_regex_unref (r);
+
+    return bands;
+}
+
+static void
+get_current_bands_done (MMAtSerialPort *port,
+                        GString *response,
+                        GError *error,
+                        gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    MMModemGsmBand mm_band = MM_MODEM_GSM_BAND_ANY;
+    GSList *bands;
+    Band *b;
+    GSList *iter;
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    if (error) {
+        info->error = g_error_copy (error);
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    bands = build_bands_info (response->str, FALSE);
+    if (!bands) {
+        mm_callback_info_set_result (info, GUINT_TO_POINTER (MM_MODEM_GSM_BAND_UNKNOWN), NULL);
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    for (iter = bands; iter; iter = g_slist_next (iter)) {
+        b = iter->data;
+        if (b->enabled)
+            mm_band |= b->band;
+    }
+    g_slist_foreach (bands, (GFunc) band_free, NULL);
+    g_slist_free (bands);
+
+    mm_callback_info_set_result (info, GUINT_TO_POINTER (mm_band), NULL);
+    mm_callback_info_schedule (info);
+}
+
+void
+mm_modem_icera_get_current_bands (MMModemIcera *self,
+                                  MMModemUIntFn callback,
+                                  gpointer user_data)
+{
+    MMAtSerialPort *port;
+    MMCallbackInfo *info;
+
+    info = mm_callback_info_uint_new (MM_MODEM (self), callback, user_data);
+
+    /* Otherwise ask the modem */
+    port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (self), &info->error);
+    if (!port) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    mm_at_serial_port_queue_command (port, "AT%IPBM?", 3, get_current_bands_done, info);
+}
+
+#define NUM_BANDS_TAG "num-bands"
+#define BAND_RESULT_TAG "band-result"
+
+static void
+get_one_supported_band_done (MMAtSerialPort *port,
+                             GString *response,
+                             GError *error,
+                             gpointer user_data)
+{
+    Band *b = user_data;
+    MMCallbackInfo *info = b->data;
+    MMModemGsmBand mm_bands = MM_MODEM_GSM_BAND_UNKNOWN;
+    guint num;
+
+    if (mm_callback_info_check_modem_removed (info) == FALSE) {
+        /* Update supported bands list */
+        mm_bands = GPOINTER_TO_UINT (mm_callback_info_get_data (info, BAND_RESULT_TAG));
+        if (error == NULL) {
+            mm_bands |= b->band;
+            mm_callback_info_set_data (info, BAND_RESULT_TAG, GUINT_TO_POINTER (mm_bands), NULL);
+        }
+
+        num = GPOINTER_TO_UINT (mm_callback_info_get_data (info, NUM_BANDS_TAG)) - 1;
+        mm_callback_info_set_data (info, NUM_BANDS_TAG, GUINT_TO_POINTER (num), NULL);
+        if (num == 0) {
+            /* All done */
+            mm_callback_info_set_result (info, GUINT_TO_POINTER (mm_bands), NULL);
+            mm_callback_info_schedule (info);
+        }
+    }
+    band_free (b);
+}
+
+static void
+get_supported_bands_done (MMAtSerialPort *port,
+                          GString *response,
+                          GError *error,
+                          gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    GSList *bands, *iter;
+    Band *b;
+    guint i, num_commands;
+    MMModemGsmBand mm_bands = MM_MODEM_GSM_BAND_UNKNOWN;
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    if (error) {
+        info->error = g_error_copy (error);
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    bands = build_bands_info (response->str, TRUE);
+    if (!bands) {
+        mm_callback_info_set_result (info, GUINT_TO_POINTER (MM_MODEM_GSM_BAND_UNKNOWN), NULL);
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    for (iter = bands, i = 0, num_commands = 0; iter; iter = g_slist_next (iter), i++) {
+        b = iter->data;
+        b->data = info;
+
+        /* If the band is already enabled, assume it's supported by the modem,
+         * otherwise we try to disable the band (which should be a NOP since it's
+         * not currently enabled) and see if the modem reports an error.
+         */
+        if (b->enabled)
+            mm_bands |= b->band;
+        else {
+            mm_at_serial_port_queue_command (port, b->name, 10, get_one_supported_band_done, b);
+            num_commands++;
+        }
+    }
+
+    if (num_commands == 0) {
+        /* All bands enabled or no band supported; schedule callback handler */
+        mm_callback_info_set_result (info, GUINT_TO_POINTER (mm_bands), NULL);
+        mm_callback_info_schedule (info);
+
+        g_slist_foreach (bands, (GFunc) band_free, NULL);
+    } else {
+        /* Callback will be scheduled by the AT command response handler */
+        mm_callback_info_set_data (info, BAND_RESULT_TAG, GUINT_TO_POINTER (mm_bands), NULL);
+        mm_callback_info_set_data (info, NUM_BANDS_TAG, GUINT_TO_POINTER (num_commands), NULL);
+
+        /* Band list *items* are freed when the AT response comes back, but let
+         * the list itself be freed here.
+         */
+    }
+
+    g_slist_free (bands);
+}
+
+void
+mm_modem_icera_get_supported_bands (MMModemIcera *self,
+                                    MMModemUIntFn callback,
+                                    gpointer user_data)
+{
+    MMAtSerialPort *port;
+    MMCallbackInfo *info;
+
+    info = mm_callback_info_uint_new (MM_MODEM (self), callback, user_data);
+
+    /* Otherwise ask the modem */
+    port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (self), &info->error);
+    if (!port) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    /* The modems report some bands as disabled that they don't actually
+     * support enabling. Thanks Icera! So we have to try setting each
+     * band to it's current enabled/disabled value, and the modem will
+     * return an error if it doesn't support that band at all.
+     */
+    mm_at_serial_port_queue_command (port, "AT%IPBM?", 3, get_supported_bands_done, info);
+}
+
+static void
+set_band_done (MMAtSerialPort *port,
+               GString *response,
+               GError *error,
+               gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    if (error)
+        info->error = g_error_copy (error);
+
+    mm_callback_info_schedule (info);
+}
+
+void
+mm_modem_icera_set_band (MMModemIcera *self,
+                         MMModemGsmBand band,
+                         MMModemFn callback,
+                         gpointer user_data)
+{
+    MMCallbackInfo *info;
+    MMAtSerialPort *port;
+    char *command;
+    const char *icera_band;
+
+    info = mm_callback_info_new (MM_MODEM (self), callback, user_data);
+
+    port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (self), &info->error);
+    if (!port) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    /* TODO: Check how to pass more than one band in the same AT%%IPBM command */
+    if (!utils_check_for_single_value (band)) {
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL, "Cannot set more than one band.");
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    icera_band = mm_band_to_icera (band);
+    if (!icera_band) {
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL, "Invalid band.");
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    command = g_strdup_printf ("AT%%IPBM=\"%s\",1", icera_band);
+    mm_at_serial_port_queue_command (port, command, 10, set_band_done, info);
+    g_free (command);
+}
+
+/****************************************************************/
+
+static void
+get_unlock_retries_done (MMAtSerialPort *port,
+                         GString *response,
+                         GError *error,
+                         gpointer user_data)
+{
+    MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    int matched;
+    GArray *retry_counts;
+    PinRetryCount ur[4] = {
+        {"sim-pin", 0}, {"sim-puk", 0}, {"sim-pin2", 0}, {"sim-puk2", 0}
+    };
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    if (error) {
+        info->error = g_error_copy (error);
+        goto done;
+    }
+
+    matched = sscanf (response->str, "%%PINNUM: %d, %d, %d, %d",
+                      &ur[0].count, &ur[1].count, &ur[2].count, &ur[3].count);
+    if (matched == 4) {
+        if (ur[0].count > 998) {
+            info->error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                       "Invalid PIN attempts left %d", ur[0].count);
+            ur[0].count = 0;
+        }
+
+        retry_counts = g_array_sized_new (FALSE, TRUE, sizeof (PinRetryCount), 4);
+        g_array_append_vals (retry_counts, &ur, 4);
+        mm_callback_info_set_result (info, retry_counts, NULL);
+    } else {
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                           "Could not parse PIN retries results");
+    }
+
+done:
+    mm_serial_port_close (MM_SERIAL_PORT (port));
+    mm_callback_info_schedule (info);
+}
+
+void
+mm_modem_icera_get_unlock_retries (MMModemIcera *self,
+                                   MMModemArrayFn callback,
+                                   gpointer user_data)
+{
+    MMAtSerialPort *port;
+    MMCallbackInfo *info;
+
+    mm_dbg ("get_unlock_retries");
+
+    info = mm_callback_info_array_new (MM_MODEM (self), callback, user_data);
+
+    /* Ensure we have a usable port to use for the command */
+    port = mm_generic_gsm_get_best_at_port (MM_GENERIC_GSM (self), &info->error);
+    if (!port) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    /* Modem may not be enabled yet, which sometimes can't be done until
+     * the device has been unlocked.  In this case we have to open the port
+     * ourselves.
+     */
+    if (!mm_serial_port_open (MM_SERIAL_PORT (port), &info->error)) {
+        mm_callback_info_schedule (info);
+        return;
+    }
+
+    /* if the modem have not yet been enabled we need to make sure echoing is turned off */
+    mm_at_serial_port_queue_command (port, "E0", 3, NULL, NULL);
+    mm_at_serial_port_queue_command (port, "%PINNUM?", 3, get_unlock_retries_done, info);
+
+}
+
+/****************************************************************/
+
 static const char *
 get_string_property (GHashTable *properties, const char *name)
 {
@@ -930,6 +1451,7 @@ mm_modem_icera_cleanup (MMModemIcera *self)
 
     /* Clear the pending connection if necessary */
     connect_pending_done (self);
+    cleanup_configure_context (self);
 
     g_free (priv->username);
     priv->username = NULL;
