@@ -244,6 +244,9 @@ mbm_erinfo_received (MMAtSerialPort *port,
         case 2:
             act = MM_MODEM_GSM_ACCESS_TECH_HSDPA;
             break;
+        case 3:
+            act = MM_MODEM_GSM_ACCESS_TECH_HSPA;
+            break;
         default:
             break;
         }
@@ -671,9 +674,13 @@ mbm_do_connect_done (MMModemMbm *self, gboolean success)
     if (!priv->pending_connect_info)
         return;
 
-    if (success)
+    if (success) {
+        /* Explicitly remove the AT*ENAP? callback, or we may end up scheduling
+         * it with an already disposed MMCallbackInfo. */
+        mm_dbg ("disabled periodic connect attempt polling");
+        mm_callback_info_set_data (priv->pending_connect_info, "mbm-enap-poll-id", NULL, NULL);
         mm_generic_gsm_connect_complete (MM_GENERIC_GSM (self), NULL, priv->pending_connect_info);
-    else {
+    } else {
         GError *connect_error;
 
         connect_error = mm_modem_connect_error_for_code (MM_MODEM_CONNECT_ERROR_BUSY);
@@ -683,6 +690,14 @@ mbm_do_connect_done (MMModemMbm *self, gboolean success)
     priv->pending_connect_info = NULL;
 }
 
+static void
+e2nap_disconnect_done (MMModem *modem,
+                       GError *error,
+                       gpointer user_data)
+{
+    mm_info ("Disconnected because the data connection failed");
+}
+
 static void
 mbm_e2nap_received (MMAtSerialPort *port,
                     GMatchInfo *info,
@@ -698,7 +713,10 @@ mbm_e2nap_received (MMAtSerialPort *port,
 
     if (MBM_E2NAP_DISCONNECTED == state) {
         mm_dbg ("disconnected");
-        mbm_do_connect_done (MM_MODEM_MBM (user_data), FALSE);
+        if (mm_modem_get_state (MM_MODEM (user_data)) == MM_MODEM_STATE_CONNECTED)
+            mm_modem_disconnect (MM_MODEM (user_data), MM_MODEM_STATE_REASON_CONNECTION_FAILED, e2nap_disconnect_done, NULL);
+        else
+            mbm_do_connect_done (MM_MODEM_MBM (user_data), FALSE);
     } else if (MBM_E2NAP_CONNECTED == state) {
         mm_dbg ("connected");
         mbm_do_connect_done (MM_MODEM_MBM (user_data), TRUE);
@@ -938,7 +956,7 @@ mbm_get_unlock_retries (MMModemGsmCard *modem,
     g_free (command);
 
     command = g_strdup_printf ("*EPIN?");
-    mm_at_serial_port_queue_command (port, command, 3, send_epin_done, info);
+    mm_at_serial_port_queue_command (port, command, 10, send_epin_done, info);
     g_free (command);
 }
 
@@ -973,6 +991,10 @@ port_grabbed (MMGenericGsm *gsm,
         mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, NULL, NULL);
         g_regex_unref (regex);
 
+        regex = g_regex_new ("\\r\\n\\*ESTKSMS:.*\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
+        mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, NULL, NULL);
+        g_regex_unref (regex);
+
         regex = g_regex_new ("\\r\\n\\*EMWI: (\\d),(\\d).*\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
         mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, NULL, NULL);
         g_regex_unref (regex);
@@ -1046,6 +1068,31 @@ finalize (GObject *object)
     G_OBJECT_CLASS (mm_modem_mbm_parent_class)->finalize (object);
 }
 
+static void
+set_property (GObject *object,
+              guint prop_id,
+              const GValue *value,
+              GParamSpec *pspec)
+{
+    /* Do nothing... see set_property() in parent, which also does nothing */
+}
+
+static void
+get_property (GObject *object,
+              guint prop_id,
+              GValue *value,
+              GParamSpec *pspec)
+{
+    switch (prop_id) {
+    case MM_GENERIC_GSM_PROP_POWER_DOWN_CMD:
+        /* Use AT+CFUN=4 for power down. */
+        g_value_set_string (value, "+CFUN=4");
+        break;
+    default:
+        break;
+    }
+}
+
 static void
 mm_modem_mbm_class_init (MMModemMbmClass *klass)
 {
@@ -1057,6 +1104,12 @@ mm_modem_mbm_class_init (MMModemMbmClass *klass)
 
     /* Virtual methods */
     object_class->finalize = finalize;
+    object_class->get_property = get_property;
+    object_class->set_property = set_property;
+
+    g_object_class_override_property (object_class,
+                                      MM_GENERIC_GSM_PROP_POWER_DOWN_CMD,
+                                      MM_GENERIC_GSM_POWER_DOWN_CMD);
 
     gsm_class->port_grabbed = port_grabbed;
     gsm_class->ports_organized = ports_organized;
