@@ -29,11 +29,13 @@
 #include "mm-modem-icera.h"
 
 static void modem_init (MMModem *modem_class);
+static void modem_gsm_network_init (MMModemGsmNetwork *gsm_network_class);
 static void modem_icera_init (MMModemIcera *icera_class);
 static void modem_simple_init (MMModemSimple *class);
 
 G_DEFINE_TYPE_EXTENDED (MMModemSierraGsm, mm_modem_sierra_gsm, MM_TYPE_GENERIC_GSM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM, modem_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_NETWORK, modem_gsm_network_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_ICERA, modem_icera_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_SIMPLE, modem_simple_init))
 
@@ -42,6 +44,7 @@ G_DEFINE_TYPE_EXTENDED (MMModemSierraGsm, mm_modem_sierra_gsm, MM_TYPE_GENERIC_G
 typedef struct {
     guint enable_wait_id;
     gboolean has_net;
+    gboolean has_lte;
     char *username;
     char *password;
     gboolean is_icera;
@@ -53,7 +56,8 @@ mm_modem_sierra_gsm_new (const char *device,
                          const char *driver,
                          const char *plugin,
                          guint32 vendor,
-                         guint32 product)
+                         guint32 product,
+                         gboolean has_lte)
 {
     MMModem *modem;
 
@@ -68,8 +72,10 @@ mm_modem_sierra_gsm_new (const char *device,
                                       MM_MODEM_HW_VID, vendor,
                                       MM_MODEM_HW_PID, product,
                                       NULL);
-    if (modem)
+    if (modem) {
         MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem)->icera = mm_modem_icera_init_private ();
+        MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem)->has_lte = has_lte;
+    }
 
     return modem;
 }
@@ -83,6 +89,7 @@ get_allowed_mode_done (MMAtSerialPort *port,
                        gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    MMModemSierraGsmPrivate *priv;
     GRegex *r = NULL;
     GMatchInfo *match_info = NULL;
 
@@ -105,6 +112,7 @@ get_allowed_mode_done (MMAtSerialPort *port,
         goto done;
     }
 
+    priv = MM_MODEM_SIERRA_GSM_GET_PRIVATE (info->modem);
     if (g_regex_match_full (r, response->str, response->len, 0, 0, &match_info, &info->error)) {
         MMModemGsmAllowedMode mode = MM_MODEM_GSM_ALLOWED_MODE_ANY;
         char *str;
@@ -121,10 +129,28 @@ get_allowed_mode_done (MMAtSerialPort *port,
             mode = MM_MODEM_GSM_ALLOWED_MODE_2G_ONLY;
             break;
         case 3:
-            mode = MM_MODEM_GSM_ALLOWED_MODE_3G_PREFERRED;
+            /* in Sierra LTE devices, mode 3 is automatic, including LTE, no preference */
+            if (priv->has_lte)
+                mode = MM_MODEM_GSM_ALLOWED_MODE_ANY;
+            else
+                mode = MM_MODEM_GSM_ALLOWED_MODE_3G_PREFERRED;
             break;
         case 4:
-            mode = MM_MODEM_GSM_ALLOWED_MODE_2G_PREFERRED;
+            /* in Sierra LTE devices, mode 4 is automatic, including LTE, no preference */
+            if (priv->has_lte)
+                mode = MM_MODEM_GSM_ALLOWED_MODE_ANY;
+            else
+                mode = MM_MODEM_GSM_ALLOWED_MODE_2G_PREFERRED;
+            break;
+        case 5:
+            /* 2G and 3G only; but we can't represent that */
+            mode = MM_MODEM_GSM_ALLOWED_MODE_3G_PREFERRED;
+            break;
+        case 6:
+            mode = MM_MODEM_GSM_ALLOWED_MODE_4G_ONLY;
+            break;
+        case 7:
+            mode = MM_MODEM_GSM_ALLOWED_MODE_4G_PREFERRED;
             break;
         default:
             info->error = g_error_new (MM_MODEM_ERROR,
@@ -204,8 +230,11 @@ set_allowed_mode (MMGenericGsm *gsm,
     MMAtSerialPort *primary;
     char *command;
     int idx = 0;
+    MMModemSierraGsmPrivate *priv;
 
-    if (MM_MODEM_SIERRA_GSM_GET_PRIVATE (self)->is_icera) {
+    priv = MM_MODEM_SIERRA_GSM_GET_PRIVATE (self);
+
+    if (priv->is_icera) {
         mm_modem_icera_set_allowed_mode (MM_MODEM_ICERA (self), mode, callback, user_data);
         return;
     }
@@ -221,6 +250,15 @@ set_allowed_mode (MMGenericGsm *gsm,
         return;
     }
 
+    if (   priv->has_lte == FALSE
+        && (   mode == MM_MODEM_GSM_ALLOWED_MODE_4G_ONLY
+            || mode == MM_MODEM_GSM_ALLOWED_MODE_4G_PREFERRED)) {
+        g_set_error_literal (&info->error, MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                             "4G allowed modes requested, but modem does not support 4G");
+        mm_callback_info_schedule (info);
+        return;
+    }
+
     switch (mode) {
     case MM_MODEM_GSM_ALLOWED_MODE_2G_ONLY:
         idx = 2;
@@ -228,11 +266,23 @@ set_allowed_mode (MMGenericGsm *gsm,
     case MM_MODEM_GSM_ALLOWED_MODE_3G_ONLY:
         idx = 1;
         break;
+    case MM_MODEM_GSM_ALLOWED_MODE_4G_ONLY:
+        idx = 6;
+        break;
     case MM_MODEM_GSM_ALLOWED_MODE_2G_PREFERRED:
-        idx = 4;
+        if (priv->has_lte)
+            idx = 2; /* 2G preferred not supported, use 2G mode instead */
+        else
+            idx = 4;
         break;
     case MM_MODEM_GSM_ALLOWED_MODE_3G_PREFERRED:
-        idx = 3;
+        if (priv->has_lte)
+            idx = 5; /* 3G preferred not supported, use 2G/3G mode instead */
+        else
+            idx = 3;
+        break;
+    case MM_MODEM_GSM_ALLOWED_MODE_4G_PREFERRED:
+        idx = 7;
         break;
     case MM_MODEM_GSM_ALLOWED_MODE_ANY:
     default:
@@ -297,6 +347,58 @@ get_access_technology (MMGenericGsm *modem,
     mm_at_serial_port_queue_command (port, "*CNTI=0", 3, get_act_request_done, info);
 }
 
+static void
+set_band (MMModemGsmNetwork *modem,
+          MMModemGsmBand band,
+          MMModemFn callback,
+          gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    if (MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem)->is_icera)
+        mm_modem_icera_set_band (MM_MODEM_ICERA (modem), band, callback, user_data);
+    else {
+        info = mm_callback_info_new (MM_MODEM (modem), callback, user_data);
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                                           "Operation not supported");
+        mm_callback_info_schedule (info);
+    }
+}
+
+static void
+get_band (MMModemGsmNetwork *modem,
+          MMModemUIntFn callback,
+          gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    if (MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem)->is_icera)
+        mm_modem_icera_get_current_bands (MM_MODEM_ICERA (modem), callback, user_data);
+    else {
+        info = mm_callback_info_uint_new (MM_MODEM (modem), callback, user_data);
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                                           "Operation not supported");
+        mm_callback_info_schedule (info);
+    }
+}
+
+static void
+get_supported_bands (MMGenericGsm *gsm,
+                     MMModemUIntFn callback,
+                     gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    if (MM_MODEM_SIERRA_GSM_GET_PRIVATE (gsm)->is_icera)
+        mm_modem_icera_get_supported_bands (MM_MODEM_ICERA (gsm), callback, user_data);
+    else {
+        info = mm_callback_info_uint_new (MM_MODEM (gsm), callback, user_data);
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                                           "Operation not supported");
+        mm_callback_info_schedule (info);
+    }
+}
+
 static void
 get_sim_iccid_done (MMAtSerialPort *port,
                     GString *response,
@@ -391,20 +493,44 @@ icera_check_cb (MMModem *modem,
                 GError *error,
                 gpointer user_data)
 {
-    if (!error) {
-        MMModemSierraGsm *self = MM_MODEM_SIERRA_GSM (user_data);
-        MMModemSierraGsmPrivate *priv = MM_MODEM_SIERRA_GSM_GET_PRIVATE (self);
+    MMCallbackInfo *info = user_data;
 
-        if (result) {
-            priv->is_icera = TRUE;
-            g_object_set (G_OBJECT (modem),
-                          MM_MODEM_IP_METHOD, MM_MODEM_IP_METHOD_STATIC,
-                          NULL);
+    if (mm_callback_info_check_modem_removed (info))
+        return;
 
-            /* Turn on unsolicited network state messages */
-            mm_modem_icera_change_unsolicited_messages (MM_MODEM_ICERA (modem), TRUE);
+    if (!error && result) {
+        /* Turn on unsolicited network state messages */
+        MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem)->is_icera = TRUE;
+        mm_modem_icera_change_unsolicited_messages (MM_MODEM_ICERA (info->modem), TRUE);
+    }
+
+    if (info->modem)
+       MM_GENERIC_GSM_CLASS (mm_modem_sierra_gsm_parent_class)->do_enable_power_up_done (MM_GENERIC_GSM (info->modem), NULL, NULL, info);
+}
+
+static void
+ws46_done_cb (MMAtSerialPort *port,
+              GString *response,
+              GError *error,
+              gpointer user_data)
+{
+    MMCallbackInfo *info = user_data;
+
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
+    /* Ignore errors */
+    if (!error && response) {
+        if (strstr (response->str, "28") ||   /* 4G only */
+            strstr (response->str, "30") ||   /* 2G/4G */
+            strstr (response->str, "31")) {  /* 3G/4G */
+            MM_MODEM_SIERRA_GSM_GET_PRIVATE (info->modem)->has_lte = TRUE;
         }
     }
+
+    mm_modem_icera_is_icera (MM_MODEM_ICERA (info->modem), icera_check_cb, info);
 }
 
 static gboolean
@@ -414,15 +540,37 @@ sierra_enabled (gpointer user_data)
     MMGenericGsm *modem;
     MMModemSierraGsmPrivate *priv;
 
-    /* Make sure we don't use an invalid modem that may have been removed */
-    if (info->modem) {
-        modem = MM_GENERIC_GSM (info->modem);
-        priv = MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem);
-        priv->enable_wait_id = 0;
-        mm_modem_icera_is_icera (MM_MODEM_ICERA (modem), icera_check_cb, MM_MODEM_SIERRA_GSM (modem));
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return FALSE;
 
-        MM_GENERIC_GSM_CLASS (mm_modem_sierra_gsm_parent_class)->do_enable_power_up_done (modem, NULL, NULL, info);
+    modem = MM_GENERIC_GSM (info->modem);
+    priv = MM_MODEM_SIERRA_GSM_GET_PRIVATE (modem);
+    priv->enable_wait_id = 0;
+
+    if (priv->has_lte)
+        mm_modem_icera_is_icera (MM_MODEM_ICERA (modem), icera_check_cb, info);
+    else {
+        MMAtSerialPort *primary;
+
+        /* Some modems (e.g. Sierra Wireless MC7710 or ZTE MF820D) won't report LTE
+         * capabilities even if they have them. So just run AT+WS46=? as well to see
+         * if the current supported modes includes any LTE-specific mode.
+         * This is not a big deal, as the AT+WS46=? command is a test command with a
+         * cache-able result.
+         *
+         * E.g.:
+         *  AT+WS46=?
+         *   +WS46: (12,22,25,28,29)
+         *   OK
+         *
+         */
+        primary = mm_generic_gsm_get_at_port (MM_GENERIC_GSM (info->modem), MM_AT_PORT_FLAG_PRIMARY);
+        g_assert (primary);
+        mm_at_serial_port_queue_command (primary, "+WS46=?", 3, ws46_done_cb, info);
     }
+
     return FALSE;
 }
 
@@ -434,6 +582,7 @@ real_do_enable_power_up_done (MMGenericGsm *gsm,
 {
     MMModemSierraGsmPrivate *priv = MM_MODEM_SIERRA_GSM_GET_PRIVATE (gsm);
     char *driver = NULL;
+    guint seconds = 10;
 
     if (error) {
         /* Chain up to parent */
@@ -441,17 +590,28 @@ real_do_enable_power_up_done (MMGenericGsm *gsm,
         return;
     }
 
-    /* Old Sierra devices (like the PCMCIA-based 860) return OK on +CFUN=1 right
-     * away but need some time to finish initialization.  Anything driven by
-     * 'sierra' is new enough to need no delay.
-     */
-    g_object_get (G_OBJECT (gsm), MM_MODEM_DRIVER, &driver, NULL);
-    if (g_strcmp0 (driver, "sierra") == 0)
+    if (response == NULL) {
+        /* If both error and response are NULL, that means the modem was already
+         * powered up and the power-up command was skipped.  So we don't need to
+         * wait for the modem to settle after CFUN=1 is sent.
+         */
         sierra_enabled (info);
-    else {
-        g_warn_if_fail (priv->enable_wait_id == 0);
-        priv->enable_wait_id = g_timeout_add_seconds (10, sierra_enabled, info);
+        return;
     }
+
+    /* Many Sierra devices return OK immediately in response to CFUN=1 but
+     * need some time to finish powering up, otherwise subsequent commands
+     * may return failure or even crash the modem.  Give more time for older
+     * devices like the AC860 and C885, which aren't driven by the 'sierra_net'
+     * driver.  Assume any DirectIP (ie, sierra_net) device is new enough
+     * to allow a lower timeout.
+     */
+    g_warn_if_fail (priv->enable_wait_id == 0);
+    g_object_get (G_OBJECT (gsm), MM_MODEM_DRIVER, &driver, NULL);
+    if (g_strcmp0 (driver, "sierra_net") == 0)
+        seconds = 5;
+
+    priv->enable_wait_id = g_timeout_add_seconds (seconds, sierra_enabled, info);
 }
 
 static void
@@ -523,6 +683,13 @@ port_grabbed (MMGenericGsm *gsm,
     if (MM_IS_AT_SERIAL_PORT (port)) {
         g_object_set (G_OBJECT (port), MM_PORT_CARRIER_DETECT, FALSE, NULL);
 
+        if (pflags == MM_AT_PORT_FLAG_SECONDARY) {
+            /* Built-in echo removal conflicts with the APP1 port's limited AT
+             * parser, which doesn't always prefix responses with <CR><LF>.
+             */
+            g_object_set (G_OBJECT (port), MM_AT_SERIAL_PORT_REMOVE_ECHO, FALSE, NULL);
+        }
+
         regex = g_regex_new ("\\r\\n\\+PACSP0\\r\\n", G_REGEX_RAW | G_REGEX_OPTIMIZE, 0, NULL);
         mm_at_serial_port_add_unsolicited_msg_handler (MM_AT_SERIAL_PORT (port), regex, NULL, NULL, NULL);
         g_regex_unref (regex);
@@ -593,7 +760,7 @@ auth_done (MMAtSerialPort *port,
 
     /* Activate data on the net port */
     command = g_strdup_printf ("!SCACT=1,%d", cid);
-    mm_at_serial_port_queue_command (port, command, 3, net_activate_done, info);
+    mm_at_serial_port_queue_command (port, command, 10, net_activate_done, info);
     g_free (command);
 }
 
@@ -854,6 +1021,18 @@ get_icera_private (MMModemIcera *icera)
     return MM_MODEM_SIERRA_GSM_GET_PRIVATE (icera)->icera;
 }
 
+static char *
+get_icera_call_control_cmd (MMModemIcera *icera, guint32 cid, gboolean activate)
+{
+    /* Sierra devices with Icera chipsets (USB 305/AT&T Lightning) still
+     * support Sierra call control commands, and using these allow DHCP
+     * to be used instead of static addressing.  The USB305 sometimes has
+     * problems with addressing when using the Icera commands, but the Sierra
+     * ones work fine.
+     */
+    return g_strdup_printf ("!SCACT=%d,%d", activate ? 1 : 0, cid);
+}
+
 /*****************************************************************************/
 
 static void
@@ -868,6 +1047,14 @@ static void
 modem_icera_init (MMModemIcera *icera)
 {
     icera->get_private = get_icera_private;
+    icera->get_call_control_cmd = get_icera_call_control_cmd;
+}
+
+static void
+modem_gsm_network_init (MMModemGsmNetwork *class)
+{
+    class->set_band = set_band;
+    class->get_band = get_band;
 }
 
 static void
@@ -895,6 +1082,35 @@ dispose (GObject *object)
     clear_user_pass (self);
 }
 
+static void
+set_property (GObject *object,
+              guint prop_id,
+              const GValue *value,
+              GParamSpec *pspec)
+{
+    /* Do nothing... see set_property() in parent, which also does nothing */
+}
+
+static void
+get_property (GObject *object,
+              guint prop_id,
+              GValue *value,
+              GParamSpec *pspec)
+{
+    switch (prop_id) {
+    case MM_GENERIC_GSM_PROP_POWER_DOWN_CMD:
+        /* Use AT+CFUN=4 for power down (low power mode) */
+        g_value_set_string (value, "+CFUN=4");
+        break;
+    case MM_GENERIC_GSM_PROP_POWER_UP_CMD:
+        /* Use AT+CFUN=1,0 for power up, to avoid reset that +cfun=1 can trigger in some modems */
+        g_value_set_string (value, "+CFUN=1,0");
+        break;
+    default:
+        break;
+    }
+}
+
 static void
 mm_modem_sierra_gsm_class_init (MMModemSierraGsmClass *klass)
 {
@@ -905,6 +1121,17 @@ mm_modem_sierra_gsm_class_init (MMModemSierraGsmClass *klass)
     g_type_class_add_private (object_class, sizeof (MMModemSierraGsmPrivate));
 
     object_class->dispose = dispose;
+    object_class->get_property = get_property;
+    object_class->set_property = set_property;
+
+    g_object_class_override_property (object_class,
+                                      MM_GENERIC_GSM_PROP_POWER_DOWN_CMD,
+                                      MM_GENERIC_GSM_POWER_DOWN_CMD);
+
+    g_object_class_override_property (object_class,
+                                      MM_GENERIC_GSM_PROP_POWER_UP_CMD,
+                                      MM_GENERIC_GSM_POWER_UP_CMD);
+
     gsm_class->port_grabbed = port_grabbed;
     gsm_class->do_enable_power_up_check_needed = do_enable_power_up_check_needed;
     gsm_class->do_enable_power_up_done = real_do_enable_power_up_done;
@@ -913,5 +1140,6 @@ mm_modem_sierra_gsm_class_init (MMModemSierraGsmClass *klass)
     gsm_class->get_access_technology = get_access_technology;
     gsm_class->get_sim_iccid = get_sim_iccid;
     gsm_class->do_disconnect = do_disconnect;
+    gsm_class->get_supported_bands = get_supported_bands;
 }
 
