@@ -32,9 +32,11 @@ static void modem_init (MMModem *modem_class);
 static void modem_icera_init (MMModemIcera *icera_class);
 static void modem_simple_init (MMModemSimple *simple_class);
 static void modem_gsm_ussd_init (MMModemGsmUssd *ussd_class);
+static void modem_gsm_network_init (MMModemGsmNetwork *gsm_network_class);
 
 G_DEFINE_TYPE_EXTENDED (MMModemZte, mm_modem_zte, MM_TYPE_GENERIC_GSM, 0,
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM, modem_init)
+                        G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_NETWORK, modem_gsm_network_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_ICERA, modem_icera_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_SIMPLE, modem_simple_init)
                         G_IMPLEMENT_INTERFACE (MM_TYPE_MODEM_GSM_USSD, modem_gsm_ussd_init)
@@ -138,7 +140,10 @@ get_allowed_mode_done (MMAtSerialPort *port,
         pref_acq = atoi (str);
         g_free (str);
 
-        if (cm_mode < 0 || cm_mode > 2 || pref_acq < 0 || pref_acq > 2) {
+        if (   cm_mode < 0
+            || (cm_mode > 2 && cm_mode != 6)
+            || pref_acq < 0
+            || (pref_acq > 2 && pref_acq != 6)) {
             info->error = g_error_new (MM_MODEM_ERROR,
                                        MM_MODEM_ERROR_GENERAL,
                                        "Failed to parse the allowed mode response: '%s'",
@@ -146,17 +151,22 @@ get_allowed_mode_done (MMAtSerialPort *port,
             goto done;
         }
 
-        if (cm_mode == 0) {  /* Both 2G and 3G allowed */
+        if (cm_mode == 0) {
+            /* 2G, 3G and LTE allowed. For LTE modems, no 2G/3G preference supported. */
             if (pref_acq == 0)
                 mode = MM_MODEM_GSM_ALLOWED_MODE_ANY;
             else if (pref_acq == 1)
                 mode = MM_MODEM_GSM_ALLOWED_MODE_2G_PREFERRED;
             else if (pref_acq == 2)
                 mode = MM_MODEM_GSM_ALLOWED_MODE_3G_PREFERRED;
+            else if (pref_acq == 6)
+                mode = MM_MODEM_GSM_ALLOWED_MODE_4G_PREFERRED;
         } else if (cm_mode == 1) /* GSM only */
             mode = MM_MODEM_GSM_ALLOWED_MODE_2G_ONLY;
         else if (cm_mode == 2) /* WCDMA only */
             mode = MM_MODEM_GSM_ALLOWED_MODE_3G_ONLY;
+        else if (cm_mode == 6) /* LTE only */
+            mode = MM_MODEM_GSM_ALLOWED_MODE_4G_ONLY;
 
         mm_callback_info_set_result (info, GUINT_TO_POINTER (mode), NULL);
     }
@@ -247,6 +257,10 @@ set_allowed_mode (MMGenericGsm *gsm,
         cm_mode = 2;
         pref_acq = 0;
         break;
+    case MM_MODEM_GSM_ALLOWED_MODE_4G_ONLY:
+        cm_mode = 6;
+        pref_acq = 0;
+        break;
     case MM_MODEM_GSM_ALLOWED_MODE_2G_PREFERRED:
         cm_mode = 0;
         pref_acq = 1;
@@ -255,6 +269,11 @@ set_allowed_mode (MMGenericGsm *gsm,
         cm_mode = 0;
         pref_acq = 2;
         break;
+    case MM_MODEM_GSM_ALLOWED_MODE_4G_PREFERRED:
+        /* ZTE modems seen until now do not support LTE preference, so use any mode instead */
+        cm_mode = 0;
+        pref_acq = 0;
+        break;
     case MM_MODEM_GSM_ALLOWED_MODE_ANY:
     default:
         break;
@@ -334,6 +353,58 @@ do_disconnect (MMGenericGsm *gsm,
         MM_GENERIC_GSM_CLASS (mm_modem_zte_parent_class)->do_disconnect (gsm, cid, callback, user_data);
 }
 
+static void
+set_band (MMModemGsmNetwork *modem,
+          MMModemGsmBand band,
+          MMModemFn callback,
+          gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    if (MM_MODEM_ZTE_GET_PRIVATE (modem)->is_icera)
+        mm_modem_icera_set_band (MM_MODEM_ICERA (modem), band, callback, user_data);
+    else {
+        info = mm_callback_info_new (MM_MODEM (modem), callback, user_data);
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                                           "Operation not supported");
+        mm_callback_info_schedule (info);
+    }
+}
+
+static void
+get_band (MMModemGsmNetwork *modem,
+          MMModemUIntFn callback,
+          gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    if (MM_MODEM_ZTE_GET_PRIVATE (modem)->is_icera)
+        mm_modem_icera_get_current_bands (MM_MODEM_ICERA (modem), callback, user_data);
+    else {
+        info = mm_callback_info_uint_new (MM_MODEM (modem), callback, user_data);
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                                           "Operation not supported");
+        mm_callback_info_schedule (info);
+    }
+}
+
+static void
+get_supported_bands (MMGenericGsm *gsm,
+                     MMModemUIntFn callback,
+                     gpointer user_data)
+{
+    MMCallbackInfo *info;
+
+    if (MM_MODEM_ZTE_GET_PRIVATE (gsm)->is_icera)
+        mm_modem_icera_get_supported_bands (MM_MODEM_ICERA (gsm), callback, user_data);
+    else {
+        info = mm_callback_info_uint_new (MM_MODEM (gsm), callback, user_data);
+        info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_OPERATION_NOT_SUPPORTED,
+                                           "Operation not supported");
+        mm_callback_info_schedule (info);
+    }
+}
+
 /*****************************************************************************/
 /*    Modem class override functions                                         */
 /*****************************************************************************/
@@ -730,6 +801,13 @@ modem_icera_init (MMModemIcera *icera)
     icera->get_private = get_icera_private;
 }
 
+static void
+modem_gsm_network_init (MMModemGsmNetwork *class)
+{
+    class->set_band = set_band;
+    class->get_band = get_band;
+}
+
 static void
 modem_simple_init (MMModemSimple *class)
 {
@@ -788,4 +866,5 @@ mm_modem_zte_class_init (MMModemZteClass *klass)
     gsm_class->set_allowed_mode = set_allowed_mode;
     gsm_class->get_allowed_mode = get_allowed_mode;
     gsm_class->get_access_technology = get_access_technology;
+    gsm_class->get_supported_bands = get_supported_bands;
 }
