@@ -49,6 +49,7 @@ mm_plugin_create (void)
 
 #define TAG_HUAWEI_PCUI_PORT "huawei-pcui-port"
 #define TAG_HUAWEI_MODEM_PORT "huawei-modem-port"
+#define TAG_HUAWEI_NDIS_PORT "huawei-ndis-port"
 #define TAG_HUAWEI_DIAG_PORT "huawei-diag-port"
 #define TAG_GETPORTMODE_SUPPORTED "getportmode-supported"
 
@@ -114,7 +115,8 @@ getportmode_response_cb (MMPluginBaseSupportsTask *task,
         MMPlugin *plugin = mm_plugin_base_supports_task_get_plugin (task);
 
         cache_port_mode (plugin, response->str, "PCUI:", TAG_HUAWEI_PCUI_PORT);
-        cache_port_mode (plugin, response->str, "MDM:", TAG_HUAWEI_MODEM_PORT);
+        cache_port_mode (plugin, response->str, "MDM:",  TAG_HUAWEI_MODEM_PORT);
+        cache_port_mode (plugin, response->str, "NDIS:", TAG_HUAWEI_NDIS_PORT);
         cache_port_mode (plugin, response->str, "DIAG:", TAG_HUAWEI_DIAG_PORT);
 
         g_object_set_data (G_OBJECT (plugin), TAG_GETPORTMODE_SUPPORTED, GUINT_TO_POINTER (1));
@@ -140,6 +142,9 @@ curc_response_cb (MMPluginBaseSupportsTask *task,
     return FALSE;
 }
 
+#define TAG_DEFER_COUNTS_PREFIX "huawei-defer-counts"
+#define MAX_DEFERS 5
+
 static MMPluginSupportsResult
 supports_port (MMPluginBase *base,
                MMModem *existing,
@@ -182,8 +187,31 @@ supports_port (MMPluginBase *base,
      * probing other interfaces until we've got if0, at which point we allow
      * the other ports to be probed too.
      */
-    if (!existing && usbif != 0)
-        return MM_PLUGIN_SUPPORTS_PORT_DEFER;
+    if (!existing && usbif != 0) {
+        gchar *tag;
+        guint n_deferred;
+
+        /* We need to defer the probing as usbif 0 wasn't probed yet. We need to
+         * protect against the case of not having usbif 0 as an AT port, and we
+         * do that by limiting the number of times we defer the probing. */
+
+        tag = g_strdup_printf ("%s-%s/%s", TAG_DEFER_COUNTS_PREFIX, subsys, name);
+        /* First time requested will be 0 */
+        n_deferred = GPOINTER_TO_UINT (g_object_get_data (G_OBJECT (base), tag));
+        if (n_deferred < MAX_DEFERS) {
+            /* Update defer count */
+            g_object_set_data (G_OBJECT (base), tag, GUINT_TO_POINTER (n_deferred + 1));
+            g_free (tag);
+            return MM_PLUGIN_SUPPORTS_PORT_DEFER;
+        }
+
+        mm_dbg ("(%s): no longer waiting for usbif 0, will launch probing in interface (%s/%s)",
+                mm_plugin_get_name (MM_PLUGIN (base)), subsys, name);
+        g_free (tag);
+
+        /* Clear tag */
+        g_object_set_data (G_OBJECT (base), tag, NULL);
+    }
 
     /* Check if a previous probing was already launched in this port */
     if (mm_plugin_base_supports_task_propagate_cached (task)) {
@@ -268,6 +296,10 @@ grab_port (MMPluginBase *base,
         pflags = MM_AT_PORT_FLAG_PRIMARY;
     else if (usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (base), TAG_HUAWEI_MODEM_PORT)))
         pflags = MM_AT_PORT_FLAG_PPP;
+    else if (!g_object_get_data (G_OBJECT (base), TAG_HUAWEI_MODEM_PORT) &&
+             usbif + 1 == GPOINTER_TO_INT (g_object_get_data (G_OBJECT (base), TAG_HUAWEI_NDIS_PORT)))
+        /* If NDIS reported only instead of MDM, use it */
+        pflags = MM_AT_PORT_FLAG_PPP;
     else if (!g_object_get_data (G_OBJECT (base), TAG_GETPORTMODE_SUPPORTED)) {
         /* If GETPORTMODE is not supported, we assume usbif 0 is the modem port */
         if ((usbif == 0) && (ptype == MM_PORT_TYPE_AT)) {
