@@ -32,8 +32,7 @@
 #include "libqcdm/src/commands.h"
 #include "libqcdm/src/errors.h"
 #include "mm-log.h"
-
-#define MM_GENERIC_CDMA_PREV_STATE_TAG "prev-state"
+#include "mm-utils.h"
 
 typedef enum {
     RM_PROTO_ASYNC = 0,
@@ -56,6 +55,8 @@ static void update_enabled_state (MMGenericCdma *self,
                                   gboolean stay_connected,
                                   MMModemStateReason reason);
 
+static guint32 get_composite_quality (MMGenericCdma *self);
+
 static void modem_init (MMModem *modem_class);
 static void modem_cdma_init (MMModemCdma *cdma_class);
 static void modem_simple_init (MMModemSimple *class);
@@ -74,6 +75,7 @@ typedef struct {
     gboolean evdo_rev0;
     gboolean evdo_revA;
     gboolean reg_try_css;
+    gboolean reg_try_cad;
     gboolean has_spservice;
     gboolean has_speri;
 
@@ -97,6 +99,7 @@ typedef struct {
     MMAtSerialPort *secondary;
     MMQcdmSerialPort *qcdm;
     MMPort *data;
+    gboolean data_opened_at_connect;
 } MMGenericCdmaPrivate;
 
 enum {
@@ -104,6 +107,7 @@ enum {
     PROP_EVDO_REV0,
     PROP_EVDO_REVA,
     PROP_REG_TRY_CSS,
+    PROP_REG_TRY_CAD,
     LAST_PROP
 };
 
@@ -230,85 +234,56 @@ owns_port (MMModem *modem, const char *subsys, const char *name)
     return !!mm_modem_base_get_port (MM_MODEM_BASE (modem), subsys, name);
 }
 
-MMPort *
-mm_generic_cdma_grab_port (MMGenericCdma *self,
-                           const char *subsys,
-                           const char *name,
-                           MMPortType suggested_type,
-                           gpointer user_data,
-                           GError **error)
+static void
+port_grabbed (MMModemBase *base,
+              MMPort *port,
+              MMAtPortFlags at_pflags,
+              gpointer user_data)
 {
-    MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
-    MMPortType ptype = MM_PORT_TYPE_IGNORED;
-    MMPort *port;
-
-    g_return_val_if_fail (!strcmp (subsys, "net") || !strcmp (subsys, "tty"), FALSE);
-    if (priv->primary)
-        g_return_val_if_fail (suggested_type != MM_PORT_TYPE_PRIMARY, FALSE);
-
-    if (!strcmp (subsys, "tty")) {
-        if (suggested_type != MM_PORT_TYPE_UNKNOWN)
-            ptype = suggested_type;
-        else {
-            if (!priv->primary)
-                ptype = MM_PORT_TYPE_PRIMARY;
-            else if (!priv->secondary)
-                ptype = MM_PORT_TYPE_SECONDARY;
-        }
-    }
-
-    port = mm_modem_base_add_port (MM_MODEM_BASE (self), subsys, name, ptype);
-    if (!port) {
-        g_warn_if_fail (port != NULL);
-        return NULL;
-    }
+    MMGenericCdma *self = MM_GENERIC_CDMA (base);
 
     if (MM_IS_AT_SERIAL_PORT (port)) {
         g_object_set (G_OBJECT (port), MM_PORT_CARRIER_DETECT, FALSE, NULL);
+        mm_at_serial_port_set_flags (MM_AT_SERIAL_PORT (port), at_pflags);
+
         mm_at_serial_port_set_response_parser (MM_AT_SERIAL_PORT (port),
                                                mm_serial_parser_v1_e1_parse,
                                                mm_serial_parser_v1_e1_new (),
                                                mm_serial_parser_v1_e1_destroy);
-
-        if (ptype == MM_PORT_TYPE_PRIMARY) {
-            priv->primary = MM_AT_SERIAL_PORT (port);
-            if (!priv->data) {
-                priv->data = port;
-                g_object_notify (G_OBJECT (self), MM_MODEM_DATA_DEVICE);
-            }
-
-            /* Get the modem's general info */
-            initial_info_check (self);
-
-            /* Get modem's ESN number */
-            initial_esn_check (self);
-
-        } else if (ptype == MM_PORT_TYPE_SECONDARY)
-            priv->secondary = MM_AT_SERIAL_PORT (port);
-    } else if (MM_IS_QCDM_SERIAL_PORT (port)) {
-        if (!priv->qcdm)
-            priv->qcdm = MM_QCDM_SERIAL_PORT (port);
-    } else if (!strcmp (subsys, "net")) {
-        /* Net device (if any) is the preferred data port */
-        if (!priv->data || MM_IS_AT_SERIAL_PORT (priv->data)) {
-            priv->data = port;
-            g_object_notify (G_OBJECT (self), MM_MODEM_DATA_DEVICE);
-            check_valid (self);
-        }
     }
 
-    return port;
+    if (MM_GENERIC_CDMA_GET_CLASS (self)->port_grabbed)
+        MM_GENERIC_CDMA_GET_CLASS (self)->port_grabbed (self, port, at_pflags, user_data);
 }
 
 static gboolean
-grab_port (MMModem *modem,
-           const char *subsys,
-           const char *name,
-           MMPortType suggested_type,
-           gpointer user_data,
-           GError **error)
+organize_ports (MMModem *modem, GError **error)
 {
-    return !!mm_generic_cdma_grab_port (MM_GENERIC_CDMA (modem), subsys, name, suggested_type, user_data, error);
+    MMGenericCdma *self = MM_GENERIC_CDMA (modem);
+    MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
+
+    if (!mm_modem_base_organize_ports (MM_MODEM_BASE (modem),
+                                       &priv->primary,
+                                       &priv->secondary,
+                                       &priv->data,
+                                       &priv->qcdm,
+                                       error))
+        return FALSE;
+
+    /* Let subclasses twiddle ports if they want */
+    if (MM_GENERIC_CDMA_GET_CLASS (self)->ports_organized)
+        MM_GENERIC_CDMA_GET_CLASS (self)->ports_organized (self, priv->primary);
+
+    g_object_notify (G_OBJECT (self), MM_MODEM_DATA_DEVICE);
+
+    /* Get the modem's general info */
+    initial_info_check (self);
+
+    /* Get modem's ESN number */
+    initial_esn_check (self);
+
+    check_valid (self);
+    return TRUE;
 }
 
 static void
@@ -346,15 +321,26 @@ release_port (MMModem *modem, const char *subsys, const char *name)
 
 MMAtSerialPort *
 mm_generic_cdma_get_at_port (MMGenericCdma *modem,
-                             MMPortType ptype)
+                             MMAtPortFlags flag)
 {
-    g_return_val_if_fail (MM_IS_GENERIC_CDMA (modem), NULL);
-    g_return_val_if_fail (ptype != MM_PORT_TYPE_UNKNOWN, NULL);
+    MMGenericCdmaPrivate *priv;
 
-    if (ptype == MM_PORT_TYPE_PRIMARY)
-        return MM_GENERIC_CDMA_GET_PRIVATE (modem)->primary;
-    else if (ptype == MM_PORT_TYPE_SECONDARY)
-        return MM_GENERIC_CDMA_GET_PRIVATE (modem)->secondary;
+    g_return_val_if_fail (MM_IS_GENERIC_CDMA (modem), NULL);
+
+    /* We only search for a single value even though it's a bitfield */
+    g_return_val_if_fail (   flag == MM_AT_PORT_FLAG_NONE
+                          || flag == MM_AT_PORT_FLAG_PRIMARY
+                          || flag == MM_AT_PORT_FLAG_SECONDARY
+                          || flag == MM_AT_PORT_FLAG_PPP, NULL);
+
+    priv = MM_GENERIC_CDMA_GET_PRIVATE (modem);
+
+    if (flag == MM_AT_PORT_FLAG_SECONDARY)
+        return priv->secondary;
+    else if (flag == MM_AT_PORT_FLAG_PRIMARY)
+        return priv->primary;
+    else if ((flag == MM_AT_PORT_FLAG_PPP) && MM_IS_AT_SERIAL_PORT (priv->data))
+        return MM_AT_SERIAL_PORT (priv->data);
 
     return NULL;
 }
@@ -410,6 +396,9 @@ mm_generic_cdma_set_1x_registration_state (MMGenericCdma *self,
                                                        priv->cdma_1x_reg_state,
                                                        priv->evdo_reg_state);
     }
+
+    /* Update signal quality too since that depends on registration state */
+    mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (self), get_composite_quality (self));
 }
 
 void
@@ -437,6 +426,9 @@ mm_generic_cdma_set_evdo_registration_state (MMGenericCdma *self,
                                                        priv->cdma_1x_reg_state,
                                                        priv->evdo_reg_state);
     }
+
+    /* Update signal quality too since that depends on registration state */
+    mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (self), get_composite_quality (self));
 }
 
 MMModemCdmaRegistrationState
@@ -592,7 +584,7 @@ crm_done (MMAtSerialPort *port,
     if (p) {
         errno = 0;
         num = strtoul (p, NULL, 10);
-        if (num >= 0 && num <= 4 && (errno == 0)) {
+        if (num <= 4 && (errno == 0)) {
             MM_GENERIC_CDMA_GET_PRIVATE (user_data)->orig_crm = (guint32) num;
             MM_GENERIC_CDMA_GET_PRIVATE (user_data)->cur_crm = (guint32) num;
         }
@@ -729,39 +721,27 @@ enable (MMModem *modem,
     mm_serial_port_flash (MM_SERIAL_PORT (priv->primary), 100, FALSE, flash_done, info);
 }
 
-static void
-disable_set_previous_state (MMModem *modem, MMCallbackInfo *info)
-{
-    MMModemState prev_state;
-
-    /* Reset old state since the operation failed */
-    prev_state = GPOINTER_TO_UINT (mm_callback_info_get_data (info, MM_GENERIC_CDMA_PREV_STATE_TAG));
-    mm_modem_set_state (modem, prev_state, MM_MODEM_STATE_REASON_NONE);
-}
-
 static void
 disable_all_done (MMModem *modem, GError *error, gpointer user_data)
 {
     MMCallbackInfo *info = user_data;
+    MMGenericCdmaPrivate *priv;
 
     /* If the modem has already been removed, return without
      * scheduling callback */
     if (!modem || mm_callback_info_check_modem_removed (info))
         return;
 
-    if (error) {
+    priv = MM_GENERIC_CDMA_GET_PRIVATE (info->modem);
+
+    if (error)
         info->error = g_error_copy (error);
-        disable_set_previous_state (modem, info);
-    } else {
-        MMGenericCdma *self = MM_GENERIC_CDMA (info->modem);
-        MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
 
-        mm_serial_port_close_force (MM_SERIAL_PORT (priv->primary));
-        mm_modem_set_state (modem, MM_MODEM_STATE_DISABLED, MM_MODEM_STATE_REASON_NONE);
+    mm_serial_port_close_force (MM_SERIAL_PORT (priv->primary));
+    mm_modem_set_state (modem, MM_MODEM_STATE_DISABLED, MM_MODEM_STATE_REASON_NONE);
 
-        priv->cdma_1x_reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN;
-        priv->evdo_reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN;
-    }
+    priv->cdma_1x_reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN;
+    priv->evdo_reg_state = MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN;
 
     mm_callback_info_schedule (info);
 }
@@ -779,14 +759,7 @@ disable_flash_done (MMSerialPort *port,
     if (mm_callback_info_check_modem_removed (info))
         return;
 
-    if (error) {
-        info->error = g_error_copy (error);
-
-        disable_set_previous_state (info->modem, info);
-        mm_callback_info_schedule (info);
-        return;
-    }
-
+    /* Ignore serial errors */
     self = MM_GENERIC_CDMA (info->modem);
 
     if (MM_GENERIC_CDMA_GET_CLASS (self)->post_disable)
@@ -803,20 +776,12 @@ disable (MMModem *modem,
     MMGenericCdma *self = MM_GENERIC_CDMA (modem);
     MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
     MMCallbackInfo *info;
-    MMModemState state;
 
     /* Tear down any ongoing registration */
     registration_cleanup (self, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL);
 
     info = mm_callback_info_new (modem, callback, user_data);
 
-    /* Cache the previous state so we can reset it if the operation fails */
-    state = mm_modem_get_state (modem);
-    mm_callback_info_set_data (info,
-                               MM_GENERIC_CDMA_PREV_STATE_TAG,
-                               GUINT_TO_POINTER (state),
-                               NULL);
-
     /* Close auxiliary serial ports */
     if (priv->secondary)
         mm_serial_port_close_force (MM_SERIAL_PORT (priv->secondary));
@@ -872,12 +837,31 @@ connect (MMModem *modem,
     MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (modem);
     MMCallbackInfo *info;
     char *command;
+    MMAtSerialPort *dial_port;
+
+    info = mm_callback_info_new (modem, callback, user_data);
+
+    /* Dial port might not be the primary port*/
+    priv->data_opened_at_connect = FALSE;
+    dial_port = priv->primary;
+    if (MM_IS_AT_SERIAL_PORT (priv->data)) {
+        dial_port = MM_AT_SERIAL_PORT (priv->data);
+
+        if (!mm_serial_port_open (MM_SERIAL_PORT (dial_port), &info->error)) {
+            g_warning ("%s: failed to open dial port: (%d) %s",
+                       __func__,
+                       info->error ? info->error->code : -1,
+                       info->error && info->error->message ? info->error->message : "(unknown)");
+            mm_callback_info_schedule (info);
+            return;
+        }
+        priv->data_opened_at_connect = TRUE;
+    }
 
     mm_modem_set_state (modem, MM_MODEM_STATE_CONNECTING, MM_MODEM_STATE_REASON_NONE);
 
-    info = mm_callback_info_new (modem, callback, user_data);
     command = g_strconcat ("DT", number, NULL);
-    mm_at_serial_port_queue_command (priv->primary, command, 90, dial_done, info);
+    mm_at_serial_port_queue_command (dial_port, command, 90, dial_done, info);
     g_free (command);
 }
 
@@ -887,24 +871,29 @@ disconnect_flash_done (MMSerialPort *port,
                        gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
-    MMModemState prev_state;
+    MMGenericCdma *self;
+    MMGenericCdmaPrivate *priv;
 
     /* If the modem has already been removed, return without
      * scheduling callback */
     if (mm_callback_info_check_modem_removed (info))
         return;
 
-    if (error) {
-        info->error = g_error_copy (error);
+    /* Ignore serial errors */
+    self = MM_GENERIC_CDMA (info->modem);
+    priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
 
-        /* Reset old state since the operation failed */
-        prev_state = GPOINTER_TO_UINT (mm_callback_info_get_data (info, MM_GENERIC_CDMA_PREV_STATE_TAG));
-        mm_modem_set_state (MM_MODEM (info->modem),
-                            prev_state,
-                            MM_MODEM_STATE_REASON_NONE);
-    } else {
-        mm_port_set_connected (MM_GENERIC_CDMA_GET_PRIVATE (info->modem)->data, FALSE);
-        update_enabled_state (MM_GENERIC_CDMA (info->modem), FALSE, MM_MODEM_STATE_REASON_NONE);
+    mm_port_set_connected (priv->data, FALSE);
+    update_enabled_state (self, FALSE, MM_MODEM_STATE_REASON_NONE);
+
+    /* Balance any open from connect(); subclasses may not use the generic
+     * class' connect function and so the dial port may not have been
+     * opened at all.
+     */
+    if (priv->data_opened_at_connect) {
+        if (MM_IS_AT_SERIAL_PORT (port))
+            mm_serial_port_close (port);
+        priv->data_opened_at_connect = FALSE;
     }
 
     mm_callback_info_schedule (info);
@@ -918,21 +907,19 @@ disconnect (MMModem *modem,
 {
     MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (modem);
     MMCallbackInfo *info;
-    MMModemState state;
+    MMAtSerialPort *dial_port;
 
     g_return_if_fail (priv->primary != NULL);
 
     info = mm_callback_info_new (modem, callback, user_data);
 
-    /* Cache the previous state so we can reset it if the operation fails */
-    state = mm_modem_get_state (modem);
-    mm_callback_info_set_data (info,
-                               MM_GENERIC_CDMA_PREV_STATE_TAG,
-                               GUINT_TO_POINTER (state),
-                               NULL);
-
     mm_modem_set_state (modem, MM_MODEM_STATE_DISCONNECTING, reason);
-    mm_serial_port_flash (MM_SERIAL_PORT (priv->primary), 1000, TRUE, disconnect_flash_done, info);
+
+    dial_port = priv->primary;
+    if (MM_IS_AT_SERIAL_PORT (priv->data))
+        dial_port = MM_AT_SERIAL_PORT (priv->data);
+
+    mm_serial_port_flash (MM_SERIAL_PORT (dial_port), 1000, TRUE, disconnect_flash_done, info);
 }
 
 static void
@@ -950,6 +937,19 @@ get_card_info (MMModem *modem,
 
 /*****************************************************************************/
 
+static guint32
+get_composite_quality (MMGenericCdma *self)
+{
+    MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
+
+    /* EVDO signal quality preferred if EVDO is registered */
+    if (   (priv->evdo_rev0 || priv->evdo_revA)
+        && (priv->evdo_reg_state > MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN)
+        && priv->evdo_quality)
+        return priv->evdo_quality;
+    return priv->cdma1x_quality;
+}
+
 void
 mm_generic_cdma_update_cdma1x_quality (MMGenericCdma *self, guint32 quality)
 {
@@ -961,7 +961,8 @@ mm_generic_cdma_update_cdma1x_quality (MMGenericCdma *self, guint32 quality)
     priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
     if (priv->cdma1x_quality != quality) {
         priv->cdma1x_quality = quality;
-        mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (self), quality);
+        mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (self),
+                                                   get_composite_quality (self));
     }
 }
 
@@ -976,7 +977,8 @@ mm_generic_cdma_update_evdo_quality (MMGenericCdma *self, guint32 quality)
     priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
     if (priv->evdo_quality != quality) {
         priv->evdo_quality = quality;
-        // FIXME: emit a signal
+        mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (self),
+                                                   get_composite_quality (self));
     }
 }
 
@@ -1027,15 +1029,15 @@ get_signal_quality_done (MMAtSerialPort *port,
                 quality = CLAMP (quality, 0, 31) * 100 / 31;
 
                 priv = MM_GENERIC_CDMA_GET_PRIVATE (info->modem);
+                mm_generic_cdma_update_cdma1x_quality (MM_GENERIC_CDMA (info->modem), quality);
+
+                /* Return composite quality, not just CDMA1x quality */
+                quality = get_composite_quality (MM_GENERIC_CDMA (info->modem));
                 mm_callback_info_set_result (info, GUINT_TO_POINTER (quality), NULL);
-                if (priv->cdma1x_quality != quality) {
-                    priv->cdma1x_quality = quality;
-                    mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (info->modem), quality);
-                }
             }
         } else
-            info->error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
-                                       "%s", "Could not parse signal quality results");
+            info->error = g_error_new_literal (MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                                               "Could not parse signal quality results");
     }
 
     mm_callback_info_schedule (info);
@@ -1095,14 +1097,12 @@ qcdm_pilot_sets_cb (MMQcdmSerialPort *port,
         best_db = CLAMP (ABS (best_db), BEST_ECIO, WORST_ECIO) - BEST_ECIO;
         quality = (guint32) (100 - (best_db * 100 / (WORST_ECIO - BEST_ECIO)));
     }
+    mm_generic_cdma_update_cdma1x_quality (MM_GENERIC_CDMA (info->modem), quality);
 
+    /* Return composite quality, not just CDMA1x quality */
+    quality = get_composite_quality (MM_GENERIC_CDMA (info->modem));
     mm_callback_info_set_result (info, GUINT_TO_POINTER (quality), NULL);
 
-    if (priv->cdma1x_quality != quality) {
-        priv->cdma1x_quality = quality;
-        mm_modem_cdma_emit_signal_quality_changed (MM_MODEM_CDMA (info->modem), quality);
-    }
-
 done:
     mm_callback_info_schedule (info);
 }
@@ -1141,26 +1141,81 @@ get_signal_quality (MMModemCdma *modem,
 }
 
 static void
-get_string_done (MMAtSerialPort *port,
-                 GString *response,
-                 GError *error,
-                 gpointer user_data)
+qcdm_esn_cb (MMQcdmSerialPort *port,
+             GByteArray *response,
+             GError *error,
+             gpointer user_data)
+{
+    MMCallbackInfo *info = user_data;
+    MMGenericCdmaPrivate *priv;
+    QcdmResult *result;
+    int err = QCDM_SUCCESS;
+    const char *esn = NULL;
+
+    if (error) {
+        info->error = g_error_copy (error);
+        goto done;
+    }
+
+    priv = MM_GENERIC_CDMA_GET_PRIVATE (info->modem);
+
+    /* Parse the response */
+    result = qcdm_cmd_esn_result ((const char *) response->data, response->len, &err);
+    if (!result) {
+        g_set_error (&info->error, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                     "Failed to parse ESN command result: %d", err);
+        goto done;
+    }
+
+    if (qcdm_result_get_string (result, QCDM_CMD_ESN_ITEM_ESN, &esn) != 0) {
+        g_set_error_literal (&info->error, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
+                             "Failed get ESN via QCDM");
+    } else
+        mm_callback_info_set_result (info, g_ascii_strdown (esn, -1), g_free);
+
+    qcdm_result_unref (result);
+
+done:
+    mm_callback_info_schedule (info);
+}
+
+static void
+get_gsn_done (MMAtSerialPort *port,
+              GString *response,
+              GError *error,
+              gpointer user_data)
 {
     MMCallbackInfo *info = (MMCallbackInfo *) user_data;
+    MMGenericCdmaPrivate *priv;
     const char *p;
+    GByteArray *esn;
 
     /* If the modem has already been removed, return without
      * scheduling callback */
     if (mm_callback_info_check_modem_removed (info))
         return;
 
-    if (error)
-        info->error = g_error_copy (error);
-    else {
-        p = mm_strip_tag (response->str, "+GSN:");
-        mm_callback_info_set_result (info, g_strdup (p), g_free);
+    priv = MM_GENERIC_CDMA_GET_PRIVATE (info->modem);
+
+    if (error) {
+        /* Fall back to QCDM if we can */
+        if (!priv->qcdm) {
+            info->error = g_error_copy (error);
+            mm_callback_info_schedule (info);
+        } else {
+            esn = g_byte_array_sized_new (20);
+            esn->len = qcdm_cmd_esn_new ((char *) esn->data, 20);
+            g_assert (esn->len);
+            mm_qcdm_serial_port_queue_command (priv->qcdm, esn, 3, qcdm_esn_cb, info);
+        }
+        return;
     }
 
+    /* AT response was OK */
+    p = mm_strip_tag (response->str, "+GSN:");
+    if (strncmp (p, "0x", 2) == 0)
+        p += 2;
+    mm_callback_info_set_result (info, g_ascii_strdown (p, -1), g_free);
     mm_callback_info_schedule (info);
 }
 
@@ -1176,11 +1231,12 @@ get_esn (MMModemCdma *modem,
 
     port = mm_generic_cdma_get_best_at_port (MM_GENERIC_CDMA (modem), &info->error);
     if (!port) {
-        mm_callback_info_schedule (info);
-        return;
+        /* Fall back to the primary port to retrieve the cached ESN */
+        port = MM_GENERIC_CDMA_GET_PRIVATE (modem)->primary;
+        g_clear_error (&info->error);
     }
 
-    mm_at_serial_port_queue_command_cached (port, "+GSN", 3, get_string_done, info);
+    mm_at_serial_port_queue_command_cached (port, "+GSN", 3, get_gsn_done, info);
 }
 
 static void
@@ -1830,6 +1886,35 @@ error:
     mm_callback_info_schedule (info);
 }
 
+static void
+reg_state_query_at (MMGenericCdma *self, MMCallbackInfo *info)
+{
+    MMGenericCdmaPrivate *priv = MM_GENERIC_CDMA_GET_PRIVATE (self);
+    MMAtSerialPort *port;
+
+    if (priv->reg_try_cad) {
+        port = mm_generic_cdma_get_best_at_port (self, &info->error);
+        if (!port) {
+            mm_callback_info_schedule (info);
+            return;
+        }
+
+        mm_at_serial_port_queue_command (port, "+CAD?", 3, get_analog_digital_done, info);
+    } else {
+        /* Just call subclass' registration function if it has one; note that
+         * MMGenericCdma's query_registration_state() will assume that we're
+         * registered on the 1X network, but the generic handler should never
+         * get called because the generic class won't set REG_TRY_CAD = FALSE.
+         */
+        g_assert (MM_GENERIC_CDMA_GET_CLASS (info->modem)->query_registration_state != real_query_registration_state);
+        MM_GENERIC_CDMA_GET_CLASS (info->modem)->query_registration_state (MM_GENERIC_CDMA (info->modem),
+                                                                           MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN,
+                                                                           MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN,
+                                                                           subclass_reg_query_done,
+                                                                           info);
+    }
+}
+
 static void
 reg_hdrstate_cb (MMQcdmSerialPort *port,
                  GByteArray *response,
@@ -1841,18 +1926,26 @@ reg_hdrstate_cb (MMQcdmSerialPort *port,
     guint32 sysmode;
     MMModemCdmaRegistrationState cdma_state = MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN;
     MMModemCdmaRegistrationState evdo_state = MM_MODEM_CDMA_REGISTRATION_STATE_UNKNOWN;
-    MMAtSerialPort *at_port;
     gboolean evdo_registered = FALSE;
+    int err = QCDM_SUCCESS;
 
-    if (error)
-        goto error;
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
 
-    sysmode = GPOINTER_TO_UINT (mm_callback_info_get_data (info, "sysmode"));
+    /* Parse the response */
+    if (!error)
+        result = qcdm_cmd_hdr_subsys_state_info_result ((const char *) response->data, response->len, &err);
 
-    /* Get HDR subsystem state to determine EVDO registration when in 1X mode */
-    result = qcdm_cmd_hdr_subsys_state_info_result ((const char *) response->data,
-                                                    response->len,
-                                                    NULL);
+    if (!result) {
+        /* Fall back to AT-based registration queries */
+        mm_dbg ("Failed to parse HDR subsys state info command result: %d", err);
+        reg_state_query_at (MM_GENERIC_CDMA (info->modem), info);
+        return;
+    }
+
+    /* Determine EVDO registration */
     if (result) {
         guint8 session_state = QCDM_CMD_HDR_SUBSYS_STATE_INFO_SESSION_STATE_CLOSED;
         guint8 almp_state = QCDM_CMD_HDR_SUBSYS_STATE_INFO_ALMP_STATE_INACTIVE;
@@ -1876,6 +1969,7 @@ reg_hdrstate_cb (MMQcdmSerialPort *port,
         qcdm_result_unref (result);
     }
 
+    sysmode = GPOINTER_TO_UINT (mm_callback_info_get_data (info, "sysmode"));
     switch (sysmode) {
     case QCDM_CMD_CM_SUBSYS_STATE_INFO_SYSTEM_MODE_CDMA:
         cdma_state = MM_MODEM_CDMA_REGISTRATION_STATE_REGISTERED;
@@ -1910,15 +2004,6 @@ reg_hdrstate_cb (MMQcdmSerialPort *port,
     set_callback_1x_state_helper (info, cdma_state);
     set_callback_evdo_state_helper (info, evdo_state);
     mm_callback_info_schedule (info);
-    return;
-
-error:
-    /* If there was some error, fall back to use +CAD like we did before QCDM */
-    at_port = mm_generic_cdma_get_best_at_port (MM_GENERIC_CDMA (info->modem), &info->error);
-    if (at_port)
-        mm_at_serial_port_queue_command (at_port, "+CAD?", 3, get_analog_digital_done, info);
-    else
-        mm_callback_info_schedule (info);
 }
 
 static void
@@ -1928,28 +2013,23 @@ reg_cmstate_cb (MMQcdmSerialPort *port,
                 gpointer user_data)
 {
     MMCallbackInfo *info = user_data;
-    MMAtSerialPort *at_port = NULL;
     QcdmResult *result = NULL;
     guint32 opmode = 0, sysmode = 0;
     int err = QCDM_SUCCESS;
 
+    /* If the modem has already been removed, return without
+     * scheduling callback */
+    if (mm_callback_info_check_modem_removed (info))
+        return;
+
     /* Parse the response */
     if (!error)
         result = qcdm_cmd_cm_subsys_state_info_result ((const char *) response->data, response->len, &err);
 
     if (!result) {
-        /* If there was some error, fall back to use +CAD like we did before QCDM */
-        if (info->modem)
-            at_port = mm_generic_cdma_get_best_at_port (MM_GENERIC_CDMA (info->modem), &info->error);
-        else {
-            g_set_error (&info->error, MM_MODEM_ERROR, MM_MODEM_ERROR_GENERAL,
-                         "Failed to parse CM subsys state info command result: %d", err);
-        }
-
-        if (at_port)
-            mm_at_serial_port_queue_command (at_port, "+CAD?", 3, get_analog_digital_done, info);
-        else
-            mm_callback_info_schedule (info);
+        /* Fall back to AT-based registration queries */
+        mm_dbg ("Failed to parse CM subsys state info command result: %d", err);
+        reg_state_query_at (MM_GENERIC_CDMA (info->modem), info);
         return;
     }
 
@@ -2013,7 +2093,7 @@ get_registration_state (MMModemCdma *modem,
         g_assert (cmstate->len);
         mm_qcdm_serial_port_queue_command (priv->qcdm, cmstate, 3, reg_cmstate_cb, info);
     } else
-        mm_at_serial_port_queue_command (port, "+CAD?", 3, get_analog_digital_done, info);
+        reg_state_query_at (MM_GENERIC_CDMA (modem), info);
 }
 
 /*****************************************************************************/
@@ -2154,7 +2234,6 @@ simple_reg_callback (MMModemCdma *modem,
     /* Fail immediately on anything but "no service" */
     if (error && !no_service_error) {
         simple_state_machine (MM_MODEM (modem), error, info);
-        g_error_free (error);
         return;
     }
 
@@ -2164,11 +2243,13 @@ simple_reg_callback (MMModemCdma *modem,
         /* Not registered yet, queue up a retry */
         priv->reg_tries++;
         if (priv->reg_tries > 15) {
-            error = g_error_new_literal (MM_MOBILE_ERROR,
-                                         MM_MOBILE_ERROR_NO_NETWORK,
-                                         "No service");
-            simple_state_machine (MM_MODEM (modem), error, info);
-            g_error_free (error);
+            GError *inner_error;
+
+            inner_error = g_error_new_literal (MM_MOBILE_ERROR,
+                                               MM_MOBILE_ERROR_NO_NETWORK,
+                                               "No service");
+            simple_state_machine (MM_MODEM (modem), inner_error, info);
+            g_error_free (inner_error);
             return;
         }
 
@@ -2333,25 +2414,6 @@ simple_connect (MMModemSimple *simple,
     g_clear_error (&error);
 }
 
-static void
-simple_free_gvalue (gpointer data)
-{
-    g_value_unset ((GValue *) data);
-    g_slice_free (GValue, data);
-}
-
-static GValue *
-simple_uint_value (guint32 i)
-{
-    GValue *val;
-
-    val = g_slice_new0 (GValue);
-    g_value_init (val, G_TYPE_UINT);
-    g_value_set_uint (val, i);
-
-    return val;
-}
-
 #define SS_HASH_TAG "simple-get-status"
 
 static void
@@ -2367,8 +2429,8 @@ simple_status_got_signal_quality (MMModem *modem,
         info->error = g_error_copy (error);
         g_warning ("Error getting signal quality: %s", error->message);
     } else {
-        properties = (GHashTable *) mm_callback_info_get_data (info, SS_HASH_TAG);
-        g_hash_table_insert (properties, "signal_quality", simple_uint_value (result));
+        properties = mm_callback_info_get_data (info, SS_HASH_TAG);
+        value_hash_add_uint (properties, "signal_quality", result);
     }
 
     mm_callback_info_schedule (info);
@@ -2397,7 +2459,7 @@ simple_get_status (MMModemSimple *simple,
                                       G_CALLBACK (callback),
                                       user_data);
 
-    properties = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, simple_free_gvalue);
+    properties = value_hash_new ();
     mm_callback_info_set_data (info, SS_HASH_TAG, properties, (GDestroyNotify) g_hash_table_unref);
     mm_modem_cdma_get_signal_quality (MM_MODEM_CDMA (simple), simple_status_got_signal_quality, info);
 }
@@ -2440,7 +2502,7 @@ static void
 modem_init (MMModem *modem_class)
 {
     modem_class->owns_port = owns_port;
-    modem_class->grab_port = grab_port;
+    modem_class->organize_ports = organize_ports;
     modem_class->release_port = release_port;
     modem_class->enable = enable;
     modem_class->disable = disable;
@@ -2497,6 +2559,9 @@ set_property (GObject *object, guint prop_id,
     case PROP_REG_TRY_CSS:
         priv->reg_try_css = g_value_get_boolean (value);
         break;
+    case PROP_REG_TRY_CAD:
+        priv->reg_try_cad = g_value_get_boolean (value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -2531,6 +2596,9 @@ get_property (GObject *object, guint prop_id,
     case PROP_REG_TRY_CSS:
         g_value_set_boolean (value, priv->reg_try_css);
         break;
+    case PROP_REG_TRY_CAD:
+        g_value_set_boolean (value, priv->reg_try_cad);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -2552,18 +2620,20 @@ dispose (GObject *object)
 }
 
 static void
-mm_generic_cdma_class_init (MMGenericCdmaClass *klass)
+mm_generic_cdma_class_init (MMGenericCdmaClass *generic_class)
 {
-    GObjectClass *object_class = G_OBJECT_CLASS (klass);
+    GObjectClass *object_class = G_OBJECT_CLASS (generic_class);
+    MMModemBaseClass *base_class = MM_MODEM_BASE_CLASS (generic_class);
 
-    mm_generic_cdma_parent_class = g_type_class_peek_parent (klass);
+    mm_generic_cdma_parent_class = g_type_class_peek_parent (generic_class);
     g_type_class_add_private (object_class, sizeof (MMGenericCdmaPrivate));
 
     /* Virtual methods */
     object_class->set_property = set_property;
     object_class->get_property = get_property;
     object_class->dispose = dispose;
-    klass->query_registration_state = real_query_registration_state;
+    base_class->port_grabbed = port_grabbed;
+    generic_class->query_registration_state = real_query_registration_state;
 
     /* Properties */
     g_object_class_override_property (object_class,
@@ -2599,5 +2669,12 @@ mm_generic_cdma_class_init (MMGenericCdmaClass *klass)
                                   " registration state.",
                                   TRUE,
                                   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
-}
 
+    g_object_class_install_property (object_class, PROP_REG_TRY_CAD,
+            g_param_spec_boolean (MM_GENERIC_CDMA_REGISTRATION_TRY_CAD,
+                                  "RegistrationTryCad",
+                                  "Use Query Analog/Digital response when checking modem"
+                                  " registration state.",
+                                  TRUE,
+                                  G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+}
