@@ -22,6 +22,7 @@
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
 #include "mm-manager.h"
+#include "mm-auth-provider.h"
 #include "mm-errors.h"
 #include "mm-plugin.h"
 #include "mm-log.h"
@@ -30,6 +31,9 @@ static gboolean impl_manager_enumerate_devices (MMManager *manager,
                                                 GPtrArray **devices,
                                                 GError **err);
 
+static void impl_manager_scan_devices (MMManager *manager,
+                                       DBusGMethodInvocation *context);
+
 static gboolean impl_manager_set_logging (MMManager *manager,
                                           const char *level,
                                           GError **error);
@@ -55,6 +59,8 @@ typedef struct {
     GSList *plugins;
     GHashTable *modems;
 
+    MMAuthProvider *authp;
+
     GHashTable *supports;
 } MMManagerPrivate;
 
@@ -117,7 +123,6 @@ load_plugin (const char *path)
     plugin = (*plugin_create_func) ();
     if (plugin) {
         g_object_weak_ref (G_OBJECT (plugin), (GWeakNotify) g_module_close, module);
-        mm_info ("Loaded plugin %s", mm_plugin_get_name (plugin));
     } else
         mm_warn ("Could not load plugin %s: initialization failed", path);
 
@@ -128,6 +133,31 @@ load_plugin (const char *path)
     return plugin;
 }
 
+static gint
+compare_plugins (const MMPlugin *plugin_a,
+                 const MMPlugin *plugin_b)
+{
+    /* The order of the plugins in the list is the same order used to check
+     * whether the plugin can manage a given modem:
+     *  - First, modems that will check vendor ID from udev.
+     *  - Then, modems that report to be sorted last (those which will check
+     *    vendor ID also from the probed ones..
+     */
+    if (mm_plugin_get_sort_last (plugin_a) &&
+        !mm_plugin_get_sort_last (plugin_b))
+        return 1;
+    if (!mm_plugin_get_sort_last (plugin_a) &&
+        mm_plugin_get_sort_last (plugin_b))
+        return -1;
+    return 0;
+}
+
+static void
+found_plugin (MMPlugin *plugin)
+{
+    mm_info ("Loaded plugin '%s'", mm_plugin_get_name (plugin));
+}
+
 static void
 load_plugins (MMManager *manager)
 {
@@ -166,10 +196,19 @@ load_plugins (MMManager *manager)
         }
     }
 
+    /* Sort last plugins that request it */
+    priv->plugins = g_slist_sort (priv->plugins, (GCompareFunc)compare_plugins);
+
     /* Make sure the generic plugin is last */
     if (generic_plugin)
         priv->plugins = g_slist_append (priv->plugins, generic_plugin);
 
+    /* Now report about all the found plugins, in the same order they will be
+     * used while checking support */
+    g_slist_foreach (priv->plugins, (GFunc)found_plugin, NULL);
+
+    mm_info ("Successfully loaded %u plugins", g_slist_length (priv->plugins));
+
     g_dir_close (dir);
 }
 
@@ -208,13 +247,36 @@ remove_modem (MMManager *manager, MMModem *modem)
     g_free (device);
 }
 
+/* Return the first outstanding supports task */
+static SupportsInfo *
+find_supports_info (MMManager *self, MMModem *modem)
+{
+    char *modem_physdev;
+    GHashTableIter iter;
+    SupportsInfo *info, *ret = NULL;
+
+    modem_physdev = mm_modem_get_device (modem);
+    g_assert (modem_physdev);
+
+    /* Check for ports that are in the process of being interrogated by plugins */
+    g_hash_table_iter_init (&iter, MM_MANAGER_GET_PRIVATE (self)->supports);
+    while (!ret && g_hash_table_iter_next (&iter, NULL, (gpointer) &info)) {
+        if (g_strcmp0 (info->physdev_path, modem_physdev) == 0)
+            ret = info;
+    }
+    g_free (modem_physdev);
+    return ret;
+}
+
 static void
 check_export_modem (MMManager *self, MMModem *modem)
 {
     MMManagerPrivate *priv = MM_MANAGER_GET_PRIVATE (self);
-    char *modem_physdev;
-    GHashTableIter iter;
-    gpointer value;
+    SupportsInfo *info;
+    static guint32 id = 0, vid = 0, pid = 0;
+    char *path, *data_device = NULL, *modem_physdev;
+    GUdevDevice *physdev;
+    const char *subsys = NULL;
 
     /* A modem is only exported to D-Bus when both of the following are true:
      *
@@ -230,63 +292,46 @@ check_export_modem (MMManager *self, MMModem *modem)
      * all other ports are already handled.  That chance is very small though.
      */
 
+    info = find_supports_info (self, modem);
+    if (info) {
+        mm_dbg ("(%s/%s): outstanding support task prevents export of %s",
+                info->subsys, info->name, info->physdev_path);
+        return;
+    }
+
+    /* Obviously don't re-export a modem, or try to export one that's not valid */
+    if (   g_object_get_data (G_OBJECT (modem), DBUS_PATH_TAG)
+        || !mm_modem_get_valid (modem))
+        return;
+
+    path = g_strdup_printf (MM_DBUS_PATH "/Modems/%d", id++);
+    dbus_g_connection_register_g_object (priv->connection, path, G_OBJECT (modem));
+    g_object_set_data_full (G_OBJECT (modem), DBUS_PATH_TAG, path, (GDestroyNotify) g_free);
+
     modem_physdev = mm_modem_get_device (modem);
     g_assert (modem_physdev);
+    mm_dbg ("Exported modem %s as %s", modem_physdev, path);
 
-    /* Check for ports that are in the process of being interrogated by plugins */
-    g_hash_table_iter_init (&iter, priv->supports);
-    while (g_hash_table_iter_next (&iter, NULL, &value)) {
-        SupportsInfo *info = value;
-
-        if (!strcmp (info->physdev_path, modem_physdev)) {
-            mm_dbg ("(%s/%s): outstanding support task prevents export of %s",
-                    info->subsys, info->name, modem_physdev);
-            goto out;
-        }
-    }
-
-    /* Already exported?  This can happen if the modem is exported and the kernel
-     * discovers another of the modem's ports.
-     */
-    if (g_object_get_data (G_OBJECT (modem), DBUS_PATH_TAG))
-        goto out;
-
-    /* No outstanding port tasks, so if the modem is valid we can export it */
-    if (mm_modem_get_valid (modem)) {
-        static guint32 id = 0, vid = 0, pid = 0;
-        char *path, *data_device = NULL;
-        GUdevDevice *physdev;
-        const char *subsys = NULL;
-
-        path = g_strdup_printf (MM_DBUS_PATH"/Modems/%d", id++);
-        dbus_g_connection_register_g_object (priv->connection, path, G_OBJECT (modem));
-        g_object_set_data_full (G_OBJECT (modem), DBUS_PATH_TAG, path, (GDestroyNotify) g_free);
-
-        mm_dbg ("Exported modem %s as %s", modem_physdev, path);
-
-        physdev = g_udev_client_query_by_sysfs_path (priv->udev, modem_physdev);
-        if (physdev)
-            subsys = g_udev_device_get_subsystem (physdev);
-
-        g_object_get (G_OBJECT (modem),
-                      MM_MODEM_DATA_DEVICE, &data_device,
-                      MM_MODEM_HW_VID, &vid,
-                      MM_MODEM_HW_PID, &pid,
-                      NULL);
-        mm_dbg ("(%s): VID 0x%04X PID 0x%04X (%s)",
-                 path, (vid & 0xFFFF), (pid & 0xFFFF),
-                 subsys ? subsys : "unknown");
-        mm_dbg ("(%s): data port is %s", path, data_device);
-        g_free (data_device);
-
-        if (physdev)
-            g_object_unref (physdev);
-
-        g_signal_emit (self, signals[DEVICE_ADDED], 0, modem);
-    }
-
-out:
+    physdev = g_udev_client_query_by_sysfs_path (priv->udev, modem_physdev);
     g_free (modem_physdev);
+    if (physdev)
+        subsys = g_udev_device_get_subsystem (physdev);
+
+    g_object_get (G_OBJECT (modem),
+                  MM_MODEM_DATA_DEVICE, &data_device,
+                  MM_MODEM_HW_VID, &vid,
+                  MM_MODEM_HW_PID, &pid,
+                  NULL);
+    mm_dbg ("(%s): VID 0x%04X PID 0x%04X (%s)",
+             path, (vid & 0xFFFF), (pid & 0xFFFF),
+             subsys ? subsys : "unknown");
+    mm_dbg ("(%s): data port is %s", path, data_device);
+    g_free (data_device);
+
+    if (physdev)
+        g_object_unref (physdev);
+
+    g_signal_emit (self, signals[DEVICE_ADDED], 0, modem);
 }
 
 static void
@@ -509,6 +554,7 @@ supports_cleanup (MMManager *self,
 {
     MMManagerPrivate *priv = MM_MANAGER_GET_PRIVATE (self);
     char *key;
+    GError *error = NULL;
 
     g_return_if_fail (subsys != NULL);
     g_return_if_fail (name != NULL);
@@ -517,15 +563,35 @@ supports_cleanup (MMManager *self,
     g_hash_table_remove (priv->supports, key);
     g_free (key);
 
+    if (modem == NULL)
+        return;
+
+    if (   (find_supports_info (self, modem) == NULL)
+        && !g_object_get_data (G_OBJECT (modem), "organized")) {
+        /* Yay, we're done with supports tasks, tell the modem to organize
+         * all its ports.  Guard against multiple organize calls though since
+         * if the kernel or udev is slow, ports may show up long after the
+         * first bunch of supports tasks is done.
+         */
+        g_object_set_data (G_OBJECT (modem), "organized", GUINT_TO_POINTER (1));
+        if (!mm_modem_organize_ports (modem, &error)) {
+            mm_err ("Failed to organize modem ports: (%d) %s",
+                    error ? error->code : -1,
+                    error && error->message ? error->message : "(unknown)");
+            g_clear_error (&error);
+            remove_modem (self, modem);
+            return;
+        }
+    }
+
     /* Each time a supports task is cleaned up, check whether the modem is
      * now completely probed/handled and should be exported to D-Bus clients.
      *
-     * IMPORTANT: this must be done after removing the supports into from
+     * IMPORTANT: this must be done after removing the supports info from
      * priv->supports since check_export_modem() searches through priv->supports
      * for outstanding supports tasks.
      */
-    if (modem)
-        check_export_modem (self, modem);
+    check_export_modem (self, modem);
 }
 
 static gboolean
@@ -637,6 +703,14 @@ supports_callback (MMPlugin *plugin,
              * supports it.
              */
             next_plugin = existing_plugin;
+        } else if (plugin != existing_plugin) {
+            /* A modem object may have been created by one of this device's other
+             * ports while this port was getting probed; now that we have a
+             * modem, just use the modem's plugin and ignore this plugin
+             * completely.  All ports of a modem must be handled by the same
+             * plugin.
+             */
+            next_plugin = NULL;
         } else {
             mm_dbg ("(%s/%s): plugin %p (%s) existing %p (%s) info->best %p (%s)",
                     info->subsys, info->name,
@@ -816,7 +890,7 @@ device_added (MMManager *manager, GUdevDevice *device)
         goto out;
     }
 
-    /* Success; now ask plugins if they can handle this port */    
+    /* Success; now ask plugins if they can handle this port */
     info = supports_info_new (manager, subsys, name, physdev_path);
     g_hash_table_insert (priv->supports, g_strdup (key), info);
 
@@ -903,24 +977,67 @@ handle_uevent (GUdevClient *client,
                gpointer user_data)
 {
     MMManager *self = MM_MANAGER (user_data);
-	const char *subsys;
+    const char *subsys;
 
-	g_return_if_fail (action != NULL);
+    g_return_if_fail (action != NULL);
 
-	/* A bit paranoid */
-	subsys = g_udev_device_get_subsystem (device);
-	g_return_if_fail (subsys != NULL);
+    /* A bit paranoid */
+    subsys = g_udev_device_get_subsystem (device);
+    g_return_if_fail (subsys != NULL);
 
-	g_return_if_fail (!strcmp (subsys, "tty") || !strcmp (subsys, "net") || !strcmp (subsys, "usb"));
+    g_return_if_fail (!strcmp (subsys, "tty") || !strcmp (subsys, "net") || !strcmp (subsys, "usb"));
 
-	/* We only care about tty/net devices when adding modem ports,
-	 * but for remove, also handle usb parent device remove events
-	 */
-	if (   (!strcmp (action, "add") || !strcmp (action, "move") || !strcmp (action, "change"))
+    /* We only care about tty/net devices when adding modem ports,
+     * but for remove, also handle usb parent device remove events
+     */
+    if (   (!strcmp (action, "add") || !strcmp (action, "move") || !strcmp (action, "change"))
         && (strcmp (subsys, "usb") != 0))
-		device_added (self, device);
-	else if (!strcmp (action, "remove"))
-		device_removed (self, device);
+        device_added (self, device);
+    else if (!strcmp (action, "remove"))
+        device_removed (self, device);
+}
+
+static void
+scan_devices_auth_cb (MMAuthRequest *req,
+                      GObject *owner,
+                      DBusGMethodInvocation *context,
+                      gpointer user_data)
+{
+    if (mm_auth_request_get_result (req) != MM_AUTH_RESULT_AUTHORIZED) {
+        GError *error;
+
+        error = g_error_new (MM_MODEM_ERROR, MM_MODEM_ERROR_AUTHORIZATION_REQUIRED,
+                             "This request requires the '%s' authorization",
+                             mm_auth_request_get_authorization (req));
+        dbus_g_method_return_error (context, error);
+        g_error_free (error);
+        return;
+    }
+
+    /* Otherwise relaunch device scan */
+    mm_manager_start (MM_MANAGER (owner));
+    dbus_g_method_return (context);
+}
+
+static void
+impl_manager_scan_devices (MMManager *manager,
+                           DBusGMethodInvocation *context)
+{
+    GError *error = NULL;
+    MMManagerPrivate *priv;
+
+    priv = MM_MANAGER_GET_PRIVATE (manager);
+    if (!mm_auth_provider_request_auth (priv->authp,
+                                        MM_AUTHORIZATION_MANAGER_CONTROL,
+                                        G_OBJECT (manager),
+                                        context,
+                                        scan_devices_auth_cb,
+                                        NULL,
+                                        NULL,
+                                        &error)) {
+        dbus_g_method_return_error (context, error);
+        g_error_free (error);
+    }
 }
 
 static gboolean
@@ -945,6 +1062,8 @@ mm_manager_start (MMManager *manager)
     g_return_if_fail (manager != NULL);
     g_return_if_fail (MM_IS_MANAGER (manager));
 
+    mm_dbg ("Starting device scan...");
+
     priv = MM_MANAGER_GET_PRIVATE (manager);
 
     devices = g_udev_client_query_by_subsystem (priv->udev, "tty");
@@ -960,6 +1079,8 @@ mm_manager_start (MMManager *manager)
         g_object_unref (G_OBJECT (iter->data));
     }
     g_list_free (devices);
+
+    mm_dbg ("Finished device scan...");
 }
 
 typedef struct {
@@ -1033,6 +1154,8 @@ mm_manager_init (MMManager *manager)
     MMManagerPrivate *priv = MM_MANAGER_GET_PRIVATE (manager);
     const char *subsys[4] = { "tty", "net", "usb", NULL };
 
+    priv->authp = mm_auth_provider_get ();
+
     priv->modems = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_object_unref);
     load_plugins (manager);
 
@@ -1048,6 +1171,8 @@ finalize (GObject *object)
 {
     MMManagerPrivate *priv = MM_MANAGER_GET_PRIVATE (object);
 
+    mm_auth_provider_cancel_for_owner (priv->authp, object);
+
     g_hash_table_destroy (priv->supports);
     g_hash_table_destroy (priv->modems);
 
