@@ -41,6 +41,46 @@ mm_strip_tag (const char *str, const char *cmd)
     return p;
 }
 
+gboolean
+mm_uint_from_match_item (GMatchInfo *info, guint32 num, guint32 *val)
+{
+    long int tmp;
+    char *str;
+    gboolean success = FALSE;
+
+    str = g_match_info_fetch (info, num);
+    g_return_val_if_fail (str != NULL, FALSE);
+
+    errno = 0;
+    tmp = strtol (str, NULL, 10);
+    if (errno == 0 && tmp >= 0 && tmp <= G_MAXUINT) {
+        *val = (guint32) tmp;
+        success = TRUE;
+    }
+    g_free (str);
+    return success;
+}
+
+gboolean
+mm_int_from_match_item (GMatchInfo *info, guint32 num, gint *val)
+{
+    long int tmp;
+    char *str;
+    gboolean success = FALSE;
+
+    str = g_match_info_fetch (info, num);
+    g_return_val_if_fail (str != NULL, FALSE);
+
+    errno = 0;
+    tmp = strtol (str, NULL, 10);
+    if (errno == 0 && tmp >= G_MININT && tmp <= G_MAXINT) {
+        *val = (gint) tmp;
+        success = TRUE;
+    }
+    g_free (str);
+    return success;
+}
+
 /*************************************************************************/
 
 static void
@@ -112,7 +152,7 @@ mm_gsm_parse_scan_response (const char *reply, GError **error)
      *       +COPS: (2,"","T-Mobile","31026",0),(1,"AT&T","AT&T","310410"),0)
      */
 
-    r = g_regex_new ("\\((\\d),([^,\\)]*),([^,\\)]*),([^,\\)]*)[\\)]?,(\\d)\\)", G_REGEX_UNGREEDY, 0, &err);
+    r = g_regex_new ("\\((\\d),\"([^\"\\)]*)\",([^,\\)]*),([^,\\)]*)[\\)]?,(\\d)\\)", G_REGEX_UNGREEDY, 0, &err);
     if (err) {
         mm_err ("Invalid regular expression: %s", err->message);
         g_error_free (err);
@@ -223,27 +263,27 @@ mm_gsm_destroy_scan_data (gpointer data)
 
 /*************************************************************************/
 
-/* +CREG: <stat>                       (GSM 07.07 CREG=1 unsolicited) */
-#define CREG1 "\\+(CREG|CGREG):\\s*(\\d{1})"
+/* +CREG: <stat>                      (GSM 07.07 CREG=1 unsolicited) */
+#define CREG1 "\\+(CREG|CGREG):\\s*0*([0-9])"
 
-/* +CREG: <n>,<stat>                   (GSM 07.07 CREG=1 solicited) */
-#define CREG2 "\\+(CREG|CGREG):\\s*(\\d{1}),\\s*(\\d{1})"
+/* +CREG: <n>,<stat>                  (GSM 07.07 CREG=1 solicited) */
+#define CREG2 "\\+(CREG|CGREG):\\s*0*([0-9]),\\s*0*([0-9])"
 
 /* +CREG: <stat>,<lac>,<ci>           (GSM 07.07 CREG=2 unsolicited) */
-#define CREG3 "\\+(CREG|CGREG):\\s*(\\d{1}),\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)"
+#define CREG3 "\\+(CREG|CGREG):\\s*0*([0-9]),\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)"
 
 /* +CREG: <n>,<stat>,<lac>,<ci>       (GSM 07.07 solicited and some CREG=2 unsolicited) */
-#define CREG4 "\\+(CREG|CGREG):\\s*(\\d{1}),\\s*(\\d{1})\\s*,\\s*([^,]*)\\s*,\\s*([^,\\s]*)"
+#define CREG4 "\\+(CREG|CGREG):\\s*0*([0-9]),\\s*0*([0-9])\\s*,\\s*([^,]*)\\s*,\\s*([^,\\s]*)"
 
 /* +CREG: <stat>,<lac>,<ci>,<AcT>     (ETSI 27.007 CREG=2 unsolicited) */
-#define CREG5 "\\+(CREG|CGREG):\\s*(\\d{1})\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*(\\d{1,2})"
+#define CREG5 "\\+(CREG|CGREG):\\s*0*([0-9])\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*0*([0-9])"
 
 /* +CREG: <n>,<stat>,<lac>,<ci>,<AcT> (ETSI 27.007 solicited and some CREG=2 unsolicited) */
-#define CREG6 "\\+(CREG|CGREG):\\s*(\\d{1}),\\s*(\\d{1})\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*(\\d{1,2})"
+#define CREG6 "\\+(CREG|CGREG):\\s*0*([0-9]),\\s*0*([0-9])\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*0*([0-9])"
 
 /* +CREG: <n>,<stat>,<lac>,<ci>,<AcT?>,<something> (Samsung Wave S8500) */
 /* '<CR><LF>+CREG: 2,1,000B,2816, B, C2816<CR><LF><CR><LF>OK<CR><LF>' */
-#define CREG7 "\\+(CREG|CGREG):\\s*(\\d{1}),\\s*(\\d{1})\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*[^,\\s]*"
+#define CREG7 "\\+(CREG|CGREG):\\s*0*([0-9]),\\s*0*([0-9])\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*[^,\\s]*"
 
 /* +CREG: <stat>,<lac>,<ci>,<AcT>,<RAC> (ETSI 27.007 v9.20 CREG=2 unsolicited with RAC) */
 #define CREG8 "\\+(CREG|CGREG):\\s*(\\d{1})\\s*,\\s*([^,\\s]*)\\s*,\\s*([^,\\s]*)\\s*,\\s*(\\d{1,2})\\s*,\\s*([^,\\s]*)"
@@ -820,6 +860,153 @@ mm_gsm_parse_cscs_support_response (const char *reply,
 
 /*************************************************************************/
 
+/* Map two letter facility codes into flag values. There are
+ * many more facilities defined (for various flavors of call
+ * barring); we only map the ones we care about. */
+static MMModemGsmFacility
+mm_gsm_string_to_facility (const char *string)
+{
+    g_return_val_if_fail (string != NULL, MM_MODEM_GSM_FACILITY_NONE);
+
+    if (!strcmp (string, "SC"))
+        return MM_MODEM_GSM_FACILITY_SIM;
+    else if (!strcmp (string, "PS"))
+        return MM_MODEM_GSM_FACILITY_PH_SIM;
+    else if (!strcmp (string, "PF"))
+        return MM_MODEM_GSM_FACILITY_PH_FSIM;
+    else if (!strcmp (string, "FD"))
+        return MM_MODEM_GSM_FACILITY_FIXED_DIALING;
+    else if (!strcmp (string, "PN"))
+        return MM_MODEM_GSM_FACILITY_NET_PERS;
+    else if (!strcmp (string, "PU"))
+        return MM_MODEM_GSM_FACILITY_NET_SUB_PERS;
+    else if (!strcmp (string, "PP"))
+        return MM_MODEM_GSM_FACILITY_PROVIDER_PERS;
+    else if (!strcmp (string, "PC"))
+        return MM_MODEM_GSM_FACILITY_CORP_PERS;
+    else
+        return MM_MODEM_GSM_FACILITY_NONE;
+
+}
+
+/*************************************************************************/
+
+char *
+mm_gsm_get_facility_name (MMModemGsmFacility facility)
+{
+    switch (facility) {
+    case MM_MODEM_GSM_FACILITY_SIM:
+        return "SC";
+    case MM_MODEM_GSM_FACILITY_PH_SIM:
+        return "PS";
+    case MM_MODEM_GSM_FACILITY_PH_FSIM:
+        return "PF";
+    case MM_MODEM_GSM_FACILITY_FIXED_DIALING:
+        return "FD";
+    case MM_MODEM_GSM_FACILITY_NET_PERS:
+        return "PN";
+    case MM_MODEM_GSM_FACILITY_NET_SUB_PERS:
+        return "PU";
+    case MM_MODEM_GSM_FACILITY_PROVIDER_PERS:
+        return "PP";
+    case MM_MODEM_GSM_FACILITY_CORP_PERS:
+        return "PC";
+    default:
+        return NULL;
+    }
+}
+
+gboolean
+mm_gsm_parse_clck_test_response (const char *reply,
+                                 MMModemGsmFacility *out_facilities)
+{
+    MMModemGsmFacility facilities = MM_MODEM_GSM_FACILITY_NONE;
+    GRegex *r;
+    GMatchInfo *match_info;
+    char *p, *str;
+    gboolean success = FALSE;
+
+    g_return_val_if_fail (reply != NULL, FALSE);
+    g_return_val_if_fail (out_facilities != NULL, FALSE);
+
+    /* the general format is:
+     *
+     * +CLCK: ("SC","AO","AI","PN")
+     */
+    p = strchr (reply, '(');
+    if (p)
+        p++;
+    else {
+        p = strchr (reply, '"');
+        if (!p)
+            return FALSE;
+    }
+
+    /* Now parse each facility */
+    r = g_regex_new ("\\s*\"([^,\\)]+)\"\\s*", 0, 0, NULL);
+    if (!r)
+        return FALSE;
+
+    if (g_regex_match_full (r, p, strlen (p), 0, 0, &match_info, NULL)) {
+        while (g_match_info_matches (match_info)) {
+            str = g_match_info_fetch (match_info, 1);
+            if (str) {
+                facilities |= mm_gsm_string_to_facility (str);
+                g_free (str);
+            }
+
+            g_match_info_next (match_info, NULL);
+            success = TRUE;
+        }
+    }
+    g_match_info_free (match_info);
+    g_regex_unref (r);
+
+    if (success)
+        *out_facilities = facilities;
+
+    return success;
+}
+
+gboolean
+mm_gsm_parse_clck_response (const char *reply, gboolean *enabled)
+{
+    GRegex *r;
+    GMatchInfo *match_info;
+    const gchar *p;
+    gchar *str;
+    gboolean success = FALSE;
+
+    g_return_val_if_fail (reply != NULL, FALSE);
+    g_return_val_if_fail (enabled != NULL, FALSE);
+
+    p = mm_strip_tag (reply, "+CLCK:");
+
+    r = g_regex_new ("\\s*([01])\\s*", 0, 0, NULL);
+    if (!r)
+        return FALSE;
+
+    if (g_regex_match (r, p, 0, &match_info)) {
+        success = TRUE;
+        str = g_match_info_fetch (match_info, 1);
+        if (str) {
+            if (*str == '0')
+                *enabled = FALSE;
+            else if (*str == '1')
+                *enabled = TRUE;
+            else
+                success = FALSE;
+
+            g_free (str);
+        }
+    }
+    g_match_info_free (match_info);
+    g_regex_unref (r);
+    return success;
+}
+
+/*************************************************************************/
+
 MMModemGsmAccessTech
 mm_gsm_string_to_access_tech (const char *string)
 {
@@ -828,7 +1015,9 @@ mm_gsm_string_to_access_tech (const char *string)
     /* Better technologies are listed first since modems sometimes say
      * stuff like "GPRS/EDGE" and that should be handled as EDGE.
      */
-    if (strcasestr (string, "HSPA+"))
+    if (strcasestr (string, "LTE") || strcasestr (string, "4G"))
+        return MM_MODEM_GSM_ACCESS_TECH_LTE;
+    else if (strcasestr (string, "HSPA+"))
         return MM_MODEM_GSM_ACCESS_TECH_HSPA_PLUS;
     else if (strcasestr (string, "HSPA"))
         return MM_MODEM_GSM_ACCESS_TECH_HSPA;
@@ -1126,4 +1315,3 @@ done:
 
     return array;
 }
-
