@@ -210,7 +210,6 @@ mm_modem_connect (MMModem *self,
 
     g_return_if_fail (MM_IS_MODEM (self));
     g_return_if_fail (callback != NULL);
-    g_return_if_fail (number != NULL);
 
     state = mm_modem_get_state (self);
     if (state >= MM_MODEM_STATE_CONNECTING) {
@@ -230,6 +229,19 @@ mm_modem_connect (MMModem *self,
         return;
     }
 
+    /* If no number given, fake it */
+    if (!number) {
+        guint32 mtype = MM_MODEM_TYPE_UNKNOWN;
+
+        g_object_get (G_OBJECT (self), MM_MODEM_TYPE, &mtype, NULL);
+        if (mtype == MM_MODEM_TYPE_CDMA)
+            number = "#777";
+        else if (mtype == MM_MODEM_TYPE_GSM)
+            number = "*99#";
+        else
+            g_assert_not_reached ();
+    }
+
     if (MM_MODEM_GET_INTERFACE (self)->connect)
         MM_MODEM_GET_INTERFACE (self)->connect (self, number, callback, user_data);
     else
@@ -251,6 +263,8 @@ get_ip4_invoke (MMCallbackInfo *info)
 
     callback (info->modem,
               GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-address")),
+              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-netmask")),
+              GPOINTER_TO_UINT (mm_callback_info_get_data (info, "ip4-gateway")),
               (GArray *) mm_callback_info_get_data (info, "ip4-dns"),
               info->error, info->user_data);
 }
@@ -293,6 +307,8 @@ value_array_add_uint (GValueArray *array, guint32 i)
 static void
 get_ip4_done (MMModem *modem,
               guint32 address,
+              guint32 netmask,
+              guint32 gateway,
               GArray *dns,
               GError *error,
               gpointer user_data)
@@ -311,7 +327,6 @@ get_ip4_done (MMModem *modem,
 
         if (dns) {
             if (dns->len > 0)
-
                 dns1 = g_array_index (dns, guint32, 0);
             if (dns->len > 1)
                 dns2 = g_array_index (dns, guint32, 1);
@@ -751,6 +766,7 @@ mm_modem_set_state (MMModem *self,
     if (new_state != old_state) {
         g_object_set (G_OBJECT (self), MM_MODEM_STATE, new_state, NULL);
         g_signal_emit_by_name (G_OBJECT (self), "state-changed", old_state, new_state, reason);
+        g_object_notify (G_OBJECT (self), MM_MODEM_STATE);
 
         dbus_path = (const char *) g_object_get_data (G_OBJECT (self), DBUS_PATH_TAG);
         if (dbus_path) {
