@@ -392,12 +392,13 @@ supports_task_dispose (GObject *object)
 
     if (priv->open_id)
         g_source_remove (priv->open_id);
-    if (priv->full_id)
-        g_source_remove (priv->full_id);
 
     if (priv->probe_id)
         g_source_remove (priv->probe_id);
     if (priv->probe_port) {
+        if (priv->full_id)
+            g_signal_handler_disconnect (priv->probe_port, priv->full_id);
+
         mm_serial_port_close (MM_SERIAL_PORT (priv->probe_port));
         g_object_unref (priv->probe_port);
     }
@@ -450,11 +451,15 @@ static struct modem_caps modem_caps[] = {
     {"+IS-856",   MM_PLUGIN_BASE_PORT_CAP_IS856},   /* Cmotech */
     {"CIS-856-A", MM_PLUGIN_BASE_PORT_CAP_IS856_A},
     {"CIS-856A",  MM_PLUGIN_BASE_PORT_CAP_IS856_A}, /* Kyocera KPC680 */
+    {"+CLTE2",    MM_PLUGIN_BASE_PORT_CAP_LTE}, /* Novatel */
+    {"+CLTE1",    MM_PLUGIN_BASE_PORT_CAP_LTE}, /* Novatel */
+    {"+CLTE",     MM_PLUGIN_BASE_PORT_CAP_LTE},
     {"+DS",       MM_PLUGIN_BASE_PORT_CAP_DS},
     {"+ES",       MM_PLUGIN_BASE_PORT_CAP_ES},
     {"+MS",       MM_PLUGIN_BASE_PORT_CAP_MS},
     {"+FCLASS",   MM_PLUGIN_BASE_PORT_CAP_FCLASS},
     {"+WIRIDIUM", MM_PLUGIN_BASE_PORT_CAP_IRIDIUM},
+    {"CDMA 1x",   MM_PLUGIN_BASE_PORT_CAP_IS707_A}, /* Huawei Data07, ATI reply */
     {NULL}
 };
 
@@ -598,6 +603,10 @@ emit_probe_result (gpointer user_data)
 
     /* Close the serial ports */
     if (task_priv->probe_port) {
+        if (task_priv->full_id) {
+            g_signal_handler_disconnect (task_priv->probe_port, task_priv->full_id);
+            task_priv->full_id = 0;
+        }
         g_object_unref (task_priv->probe_port);
         task_priv->probe_port = NULL;
     }
@@ -643,17 +652,15 @@ qcdm_verinfo_cb (MMQcdmSerialPort *port,
     MMPluginBaseSupportsTaskPrivate *priv;
     QcdmResult *result;
     int err = QCDM_SUCCESS;
-
-    /* Just the initial poke; ignore it */
-    if (!user_data)
-        return;
+    GByteArray *cmd2;
 
     task = MM_PLUGIN_BASE_SUPPORTS_TASK (user_data);
     priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
 
     if (error) {
-        /* Probably not a QCDM port */
-        goto done;
+        if (!g_error_matches (error, MM_SERIAL_ERROR, MM_SERIAL_ERROR_RESPONSE_TIMEOUT))
+            mm_dbg ("QCDM probe error: (%d) %s", error->code, error->message);
+        goto again;
     }
 
     /* Parse the response */
@@ -661,14 +668,24 @@ qcdm_verinfo_cb (MMQcdmSerialPort *port,
     if (!result) {
         g_warning ("(%s) failed to parse QCDM version info command result: %d",
                    g_udev_device_get_name (priv->port), err);
-        goto done;
+        goto again;
     }
 
     /* yay, probably a QCDM port */
     qcdm_result_unref (result);
     priv->probed_caps |= MM_PLUGIN_BASE_PORT_CAP_QCDM;
-done:
     probe_complete (task);
+    return;
+
+again:
+    cmd2 = g_object_steal_data (G_OBJECT (task), "cmd2");
+    if (cmd2) {
+        /* second try */
+        mm_qcdm_serial_port_queue_command (priv->qcdm_port, cmd2, 3, qcdm_verinfo_cb, task);
+    } else {
+        /* all done */
+        probe_complete (task);
+    }
 }
 
 static void
@@ -679,9 +696,14 @@ try_qcdm_probe (MMPluginBaseSupportsTask *task)
     GError *error = NULL;
     GByteArray *verinfo = NULL, *verinfo2;
     gint len;
+    guint8 marker = 0x7E;
 
     /* Close the AT port */
     if (priv->probe_port) {
+        if (priv->full_id) {
+            g_signal_handler_disconnect (priv->probe_port, priv->full_id);
+            priv->full_id = 0;
+        }
         mm_serial_port_close (MM_SERIAL_PORT (priv->probe_port));
         g_object_unref (priv->probe_port);
         priv->probe_port = NULL;
@@ -707,24 +729,29 @@ try_qcdm_probe (MMPluginBaseSupportsTask *task)
         return;
     }
 
-    /* Build up the probe command */
-    verinfo = g_byte_array_sized_new (50);
-    len = qcdm_cmd_version_info_new ((char *) verinfo->data, 50);
+    /* Build up the probe command; 0x7E is the frame marker, so put one at the
+     * beginning of the buffer to ensure that the device discards any AT
+     * commands that probing might have sent earlier.  Should help devices
+     * respond more quickly and speed up QCDM probing.
+     */
+    verinfo = g_byte_array_sized_new (10);
+    g_byte_array_append (verinfo, &marker, 1);
+
+    len = qcdm_cmd_version_info_new ((char *) (verinfo->data + 1), 9);
     if (len <= 0) {
         g_byte_array_free (verinfo, TRUE);
         g_warning ("(%s) failed to create QCDM version info command", name);
         probe_complete (task);
         return;
     }
-    verinfo->len = len;
+    verinfo->len = len + 1;
 
-    /* Queuing the command takes ownership over it; copy it for the second try */
+    /* Queuing the command takes ownership over it; save a copy for the second try */
     verinfo2 = g_byte_array_sized_new (verinfo->len);
     g_byte_array_append (verinfo2, verinfo->data, verinfo->len);
+    g_object_set_data_full (G_OBJECT (task), "cmd2", verinfo2, (GDestroyNotify) g_byte_array_unref);
 
-    /* Send the command twice; the ports often need to be woken up */
-    mm_qcdm_serial_port_queue_command (priv->qcdm_port, verinfo, 3, qcdm_verinfo_cb, NULL);
-    mm_qcdm_serial_port_queue_command (priv->qcdm_port, verinfo2, 3, qcdm_verinfo_cb, task);
+    mm_qcdm_serial_port_queue_command (priv->qcdm_port, verinfo, 3, qcdm_verinfo_cb, task);
 }
 
 static void
@@ -1054,11 +1081,12 @@ try_open (gpointer user_data)
     return FALSE;
 }
 
-gboolean
-mm_plugin_base_probe_port (MMPluginBase *self,
-                           MMPluginBaseSupportsTask *task,
-                           guint64 send_delay_us,
-                           GError **error)
+static gboolean
+_internal_probe_port (MMPluginBase *self,
+                      MMPluginBaseSupportsTask *task,
+                      guint64 send_delay_us,
+                      gboolean remove_echo,
+                      GError **error)
 {
     MMPluginBaseSupportsTaskPrivate *task_priv = MM_PLUGIN_BASE_SUPPORTS_TASK_GET_PRIVATE (task);
     MMAtSerialPort *serial;
@@ -1085,6 +1113,7 @@ mm_plugin_base_probe_port (MMPluginBase *self,
                   MM_SERIAL_PORT_SEND_DELAY, send_delay_us,
                   MM_PORT_CARRIER_DETECT, FALSE,
                   MM_SERIAL_PORT_SPEW_CONTROL, TRUE,
+                  MM_AT_SERIAL_PORT_REMOVE_ECHO, remove_echo,
                   NULL);
 
     mm_at_serial_port_set_response_parser (serial,
@@ -1099,6 +1128,24 @@ mm_plugin_base_probe_port (MMPluginBase *self,
     return TRUE;
 }
 
+gboolean
+mm_plugin_base_probe_port (MMPluginBase *self,
+                           MMPluginBaseSupportsTask *task,
+                           guint64 send_delay_us,
+                           GError **error)
+{
+    return _internal_probe_port (self, task, send_delay_us, TRUE, error);
+}
+
+gboolean
+mm_plugin_base_probe_port_leave_echo (MMPluginBase *self,
+                                      MMPluginBaseSupportsTask *task,
+                                      guint64 send_delay_us,
+                                      GError **error)
+{
+    return _internal_probe_port (self, task, send_delay_us, FALSE, error);
+}
+
 gboolean
 mm_plugin_base_get_cached_probe_result (MMPluginBase *self,
                                         GUdevDevice *port,
@@ -1156,6 +1203,7 @@ mm_plugin_base_get_device_ids (MMPluginBase *self,
     GUdevDevice *device = NULL, *parent = NULL;
     const char *vid = NULL, *pid = NULL, *parent_subsys;
     gboolean success = FALSE;
+    char *pci_vid = NULL, *pci_pid = NULL;
 
     g_return_val_if_fail (self != NULL, FALSE);
     g_return_val_if_fail (MM_IS_PLUGIN_BASE (self), FALSE);
@@ -1195,6 +1243,19 @@ mm_plugin_base_get_device_ids (MMPluginBase *self,
                 /* Platform devices don't usually have a VID/PID */
                 success = TRUE;
                 goto out;
+            } else if (g_str_equal (parent_subsys, "pci")) {
+                const char *pci_id;
+
+                /* We can't always rely on the model + vendor showing up on
+                 * the PCI device's child, so look at the PCI parent. PCI_ID
+                 * has the format "1931:000C".
+                 */
+                pci_id = g_udev_device_get_property (parent, "PCI_ID");
+                if (pci_id && strlen (pci_id) == 9 && pci_id[4] == ':') {
+                    vid = pci_vid = g_strdup (pci_id);
+                    pci_vid[4] = '\0';
+                    pid = pci_pid = g_strdup (pci_id + 5);
+                }
             }
         }
     }
@@ -1240,6 +1301,8 @@ out:
         g_object_unref (device);
     if (parent)
         g_object_unref (parent);
+    g_free (pci_vid);
+    g_free (pci_pid);
     return success;
 }
 
