@@ -226,6 +226,7 @@ typedef struct {
     GRegex *regex_ezx_error;
     GRegex *regex_unknown_error;
     GRegex *regex_connect_failed;
+    GRegex *regex_na;
     GRegex *regex_custom_error;
 } MMSerialParserV1;
 
@@ -239,7 +240,7 @@ mm_serial_parser_v1_new (void)
 
     parser->regex_ok = g_regex_new ("\\r\\nOK(\\r\\n)+$", flags, 0, NULL);
     parser->regex_connect = g_regex_new ("\\r\\nCONNECT.*\\r\\n", flags, 0, NULL);
-    parser->regex_sms = g_regex_new ("\\r\\n>\\s$", flags, 0, NULL);
+    parser->regex_sms = g_regex_new ("\\r\\n>\\s*$", flags, 0, NULL);
     parser->regex_cme_error = g_regex_new ("\\r\\n\\+CME ERROR:\\s*(\\d+)\\r\\n$", flags, 0, NULL);
     parser->regex_cms_error = g_regex_new ("\\r\\n\\+CMS ERROR:\\s*(\\d+)\\r\\n$", flags, 0, NULL);
     parser->regex_cme_error_str = g_regex_new ("\\r\\n\\+CME ERROR:\\s*([^\\n\\r]+)\\r\\n$", flags, 0, NULL);
@@ -247,6 +248,8 @@ mm_serial_parser_v1_new (void)
     parser->regex_ezx_error = g_regex_new ("\\r\\n\\MODEM ERROR:\\s*(\\d+)\\r\\n$", flags, 0, NULL);
     parser->regex_unknown_error = g_regex_new ("\\r\\n(ERROR)|(COMMAND NOT SUPPORT)\\r\\n$", flags, 0, NULL);
     parser->regex_connect_failed = g_regex_new ("\\r\\n(NO CARRIER)|(BUSY)|(NO ANSWER)|(NO DIALTONE)\\r\\n$", flags, 0, NULL);
+    /* Samsung Z810 may reply "NA" to report a not-available error */
+    parser->regex_na = g_regex_new ("\\r\\nNA\\r\\n", flags, 0, NULL);
 
     parser->regex_custom_successful = NULL;
     parser->regex_custom_error = NULL;
@@ -436,6 +439,19 @@ mm_serial_parser_v1_parse (gpointer data,
         }
 
         local_error = mm_modem_connect_error_for_code (code);
+        goto done;
+    }
+
+    /* NA error */
+    found = g_regex_match_full (parser->regex_na,
+                                response->str, response->len,
+                                0, 0, &match_info, NULL);
+    if (found) {
+        /* Assume NA means 'Not Allowed' :) */
+        local_error = g_error_new (MM_MOBILE_ERROR,
+                                   MM_MOBILE_ERROR_NOT_ALLOWED,
+                                   "Not Allowed");
+        goto done;
     }
 
 done:
@@ -469,6 +485,7 @@ mm_serial_parser_v1_destroy (gpointer data)
     g_regex_unref (parser->regex_ezx_error);
     g_regex_unref (parser->regex_unknown_error);
     g_regex_unref (parser->regex_connect_failed);
+    g_regex_unref (parser->regex_na);
 
     if (parser->regex_custom_successful)
         g_regex_unref (parser->regex_custom_successful);
