@@ -753,6 +753,27 @@ data_available (GIOChannel *source,
     return TRUE;
 }
 
+static void
+data_watch_enable (MMSerialPort *self, gboolean enable)
+{
+    MMSerialPortPrivate *priv = MM_SERIAL_PORT_GET_PRIVATE (self);
+
+    if (priv->watch_id) {
+        if (enable)
+            g_warn_if_fail (priv->watch_id == 0);
+
+        g_source_remove (priv->watch_id);
+        priv->watch_id = 0;
+    }
+
+    if (enable) {
+        g_return_if_fail (priv->channel != NULL);
+        priv->watch_id = g_io_add_watch (priv->channel,
+                                         G_IO_IN | G_IO_ERR | G_IO_HUP,
+                                         data_available, self);
+    }
+}
+
 static void
 port_connected (MMSerialPort *self, GParamSpec *pspec, gpointer user_data)
 {
@@ -780,6 +801,9 @@ port_connected (MMSerialPort *self, GParamSpec *pspec, gpointer user_data)
             // close the port and error out?
         }
     }
+
+    /* When connected ignore let PPP have all the data */
+    data_watch_enable (self, !connected);
 }
 
 gboolean
@@ -788,7 +812,7 @@ mm_serial_port_open (MMSerialPort *self, GError **error)
     MMSerialPortPrivate *priv;
     char *devfile;
     const char *device;
-    struct serial_struct sinfo;
+    struct serial_struct sinfo = { 0 };
     GTimeVal tv_start, tv_end;
 
     g_return_val_if_fail (MM_IS_SERIAL_PORT (self), FALSE);
@@ -847,7 +871,7 @@ mm_serial_port_open (MMSerialPort *self, GError **error)
 
     /* Don't wait for pending data when closing the port; this can cause some
      * stupid devices that don't respond to URBs on a particular port to hang
-     * for 30 seconds when probin fails.
+     * for 30 seconds when probing fails.  See GNOME bug #630670.
      */
     if (ioctl (priv->fd, TIOCGSERIAL, &sinfo) == 0) {
         sinfo.closing_wait = ASYNC_CLOSING_WAIT_NONE;
@@ -861,9 +885,7 @@ mm_serial_port_open (MMSerialPort *self, GError **error)
 
     priv->channel = g_io_channel_unix_new (priv->fd);
     g_io_channel_set_encoding (priv->channel, NULL, NULL);
-    priv->watch_id = g_io_add_watch (priv->channel,
-                                     G_IO_IN | G_IO_ERR | G_IO_HUP,
-                                     data_available, self);
+    data_watch_enable (self, TRUE);
 
     g_warn_if_fail (priv->connected_id == 0);
     priv->connected_id = g_signal_connect (self, "notify::" MM_PORT_CONNECTED,
@@ -919,21 +941,32 @@ mm_serial_port_close (MMSerialPort *self)
 
     if (priv->fd >= 0) {
         GTimeVal tv_start, tv_end;
+        struct serial_struct sinfo = { 0 };
 
         mm_info ("(%s) closing serial port...", device);
 
         mm_port_set_connected (MM_PORT (self), FALSE);
 
+        /* Paranoid: ensure our closing_wait value is still set so we ignore
+         * pending data when closing the port.  See GNOME bug #630670.
+         */
+        if (ioctl (priv->fd, TIOCGSERIAL, &sinfo) == 0) {
+            if (sinfo.closing_wait != ASYNC_CLOSING_WAIT_NONE) {
+                mm_warn ("(%s): serial port closing_wait was reset!", device);
+                sinfo.closing_wait = ASYNC_CLOSING_WAIT_NONE;
+                (void) ioctl (priv->fd, TIOCSSERIAL, &sinfo);
+            }
+        }
+
+        g_get_current_time (&tv_start);
+
         if (priv->channel) {
-            g_source_remove (priv->watch_id);
-            priv->watch_id = 0;
+            data_watch_enable (self, FALSE);
             g_io_channel_shutdown (priv->channel, TRUE, NULL);
             g_io_channel_unref (priv->channel);
             priv->channel = NULL;
         }
 
-        g_get_current_time (&tv_start);
-
         tcsetattr (priv->fd, TCSANOW, &priv->old_t);
         tcflush (priv->fd, TCIOFLUSH);
         close (priv->fd);
