@@ -56,7 +56,7 @@ test_cops_results (const char *desc,
     g_assert (results);
     g_assert (error == NULL);
 
-    g_assert (results->len == expected_results_len);
+    g_assert_cmpint (results->len, ==, expected_results_len);
 
     for (i = 0; i < results->len; i++) {
         GHashTable *entry = g_ptr_array_index (results, i);
@@ -65,31 +65,31 @@ test_cops_results (const char *desc,
 
         value = g_hash_table_lookup (entry, MM_SCAN_TAG_STATUS);
         g_assert (value);
-        g_assert (strcmp (value, expected->status) == 0);
+        g_assert_cmpstr (value, ==, expected->status);
 
         value = g_hash_table_lookup (entry, MM_SCAN_TAG_OPER_LONG);
         if (expected->oper_long) {
             g_assert (value);
-            g_assert (strcmp (value, expected->oper_long) == 0);
+            g_assert_cmpstr (value, ==, expected->oper_long);
         } else
             g_assert (value == NULL);
 
         value = g_hash_table_lookup (entry, MM_SCAN_TAG_OPER_SHORT);
         if (expected->oper_short) {
             g_assert (value);
-            g_assert (strcmp (value, expected->oper_short) == 0);
+            g_assert_cmpstr (value, ==, expected->oper_short);
         } else
             g_assert (value == NULL);
 
         value = g_hash_table_lookup (entry, MM_SCAN_TAG_OPER_NUM);
         g_assert (expected->oper_num);
         g_assert (value);
-        g_assert (strcmp (value, expected->oper_num) == 0);
+        g_assert_cmpstr (value, ==, expected->oper_num);
 
         value = g_hash_table_lookup (entry, MM_SCAN_TAG_ACCESS_TECH);
         if (expected->tech) {
             g_assert (value);
-            g_assert (strcmp (value, expected->tech) == 0);
+            g_assert_cmpstr (value, ==, expected->tech);
         } else
             g_assert (value == NULL);
     }
@@ -435,6 +435,19 @@ test_cops_response_sek600i (void *f, gpointer d)
     test_cops_results ("Sony-Ericsson K600i", reply, &expected[0], ARRAY_LEN (expected));
 }
 
+static void
+test_cops_response_samsung_z810 (void *f, gpointer d)
+{
+    /* Ensure commas within quotes don't trip up the parser */
+    const char *reply = "+COPS: (1,\"T-Mobile USA, In\",\"T-Mobile\",\"310260\",0),(1,\"AT&T\",\"AT&T\",\"310410\",0),,(0,1,2,3,4),(0,1,2)";
+    static OperEntry expected[] = {
+        { "1", "T-Mobile USA, In", "T-Mobile", "310260", "0" },
+        { "1", "AT&T", "AT&T", "310410", "0" },
+    };
+
+    test_cops_results ("Samsung Z810", reply, &expected[0], ARRAY_LEN (expected));
+}
+
 static void
 test_cops_response_gsm_invalid (void *f, gpointer d)
 {
@@ -501,6 +514,7 @@ test_creg_match (const char *test,
         GRegex *r = g_ptr_array_index (array, i);
 
         if (g_regex_match (r, reply, 0, &info)) {
+            g_print ("  matched with %d\n", i);
             regex_num = i + 1;
             break;
         }
@@ -508,6 +522,10 @@ test_creg_match (const char *test,
         info = NULL;
     }
 
+    g_print ("  regex_num (%u) == result->regex_num (%u)\n",
+             regex_num,
+             result->regex_num);
+
     g_assert (info != NULL);
     g_assert (regex_num == result->regex_num);
 
@@ -517,6 +535,9 @@ test_creg_match (const char *test,
     g_assert (state == result->state);
     g_assert (lac == result->lac);
     g_assert (ci == result->ci);
+
+    g_print ("  access_tech (%d) == result->act (%d)\n",
+             access_tech, result->act);
     g_assert (access_tech == result->act);
     g_assert (cgreg == result->cgreg);
 }
@@ -632,6 +653,16 @@ test_creg2_xu870_unsolicited_unregistered (void *f, gpointer d)
     test_creg_match ("Novatel XU870 unregistered CREG=2", FALSE, reply, data, &result);
 }
 
+static void
+test_creg2_iridium_solicited (void *f, gpointer d)
+{
+    TestData *data = (TestData *) d;
+    const char *reply = "+CREG:002,001,\"18d8\",\"ffff\"";
+    const CregResult result = { 1, 0x18D8, 0xFFFF, -1 , 4, FALSE};
+
+    test_creg_match ("Iridium, CREG=2", TRUE, reply, data, &result);
+}
+
 static void
 test_cgreg1_solicited (void *f, gpointer d)
 {
@@ -1219,45 +1250,46 @@ typedef void (*TCFunc)(void);
 
 int main (int argc, char **argv)
 {
-	GTestSuite *suite;
+    GTestSuite *suite;
     TestData *data;
     gint result;
     DevidItem *item = &devids[0];
 
-	g_test_init (&argc, &argv, NULL);
+    g_test_init (&argc, &argv, NULL);
 
-	suite = g_test_get_root ();
+    suite = g_test_get_root ();
     data = test_data_new ();
 
-	g_test_suite_add (suite, TESTCASE (test_cops_response_tm506, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_gt3gplus, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_ac881, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_gtmax36, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_ac860, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_gtm378, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_motoc, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_mf627a, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_mf627b, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_e160g, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_mercury, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_quicksilver, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_icon225, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_icon452, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_f3507g, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_f3607gw, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_mc8775, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_n80, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_e1550, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_mf622, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_e226, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_xu870, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_gtultraexpress, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_n2720, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_gobi, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_sek600i, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_tm506, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_gt3gplus, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_ac881, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_gtmax36, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_ac860, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_gtm378, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_motoc, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_mf627a, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_mf627b, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_e160g, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_mercury, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_quicksilver, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_icon225, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_icon452, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_f3507g, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_f3607gw, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_mc8775, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_n80, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_e1550, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_mf622, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_e226, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_xu870, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_gtultraexpress, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_n2720, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_gobi, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_sek600i, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_samsung_z810, NULL));
 
-	g_test_suite_add (suite, TESTCASE (test_cops_response_gsm_invalid, NULL));
-	g_test_suite_add (suite, TESTCASE (test_cops_response_umts_invalid, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_gsm_invalid, NULL));
+    g_test_suite_add (suite, TESTCASE (test_cops_response_umts_invalid, NULL));
 
     g_test_suite_add (suite, TESTCASE (test_creg1_solicited, data));
     g_test_suite_add (suite, TESTCASE (test_creg1_unsolicited, data));
@@ -1273,6 +1305,7 @@ int main (int argc, char **argv)
     g_test_suite_add (suite, TESTCASE (test_creg2_md400_unsolicited, data));
     g_test_suite_add (suite, TESTCASE (test_creg2_s8500_wave_unsolicited, data));
     g_test_suite_add (suite, TESTCASE (test_creg2_gobi_weird_solicited, data));
+    g_test_suite_add (suite, TESTCASE (test_creg2_iridium_solicited, data));
 
     g_test_suite_add (suite, TESTCASE (test_cgreg1_solicited, data));
     g_test_suite_add (suite, TESTCASE (test_cgreg1_unsolicited, data));
